/**
 * nSX2 - an OpenSource PS2 Emulator
 * Copyright (C) 2002 Muad
 *
 * homepage: http://nsx2.emulation64.com
 * email address: atreides.muaddib@caramail.com
 * 
 * If you want to contribute to the project please contact
 * me first (maybe someone is already making what you are
 * planning to do).
 *
 *
 * This program is free software; you can redistribute it and/
 * or modify it under the terms of the GNU General Public Li-
 * cence as published by the Free Software Foundation; either
 * version 2 of the Licence, or any later version.
 *
 * This program is distributed in the hope that it will be use-
 * ful, but WITHOUT ANY WARRANTY; without even the implied war-
 * ranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public Licence for more details.
 *
 * You should have received a copy of the GNU General Public
 * Licence along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
**/

#include <stdio.h>
#include <stdlib.h>

#include <SDL/SDL.h>

#include "../nSX2.h"
#include "muGUI.h"


/*
muGUI Init Fucntions
*/

int muGUI_Init ( muGUI * fgui ) {

	SDL_Surface  *video_screen;	//temp Variable			


   /* Create a 32-bit surface with the bytes of each pixel in R,G,B,A order,
       as expected by OpenGL for textures */
    Uint32 rmask, gmask, bmask, amask;

    /* SDL interprets each pixel as a 32-bit number, so our masks must depend
       on the endianness (byte order) of the machine */
	#if SDL_BYTEORDER == SDL_BIG_ENDIAN
	    rmask = 0xff000000;
	    gmask = 0x00ff0000;
	    bmask = 0x0000ff00;
	    amask = 0x000000ff;
	#else
	    rmask = 0x000000ff;
	    gmask = 0x0000ff00;
	    bmask = 0x00ff0000;
	    amask = 0xff000000;
	#endif

	
	/* Get the current Video Surface */
	video_screen = SDL_GetVideoSurface ();

	/*Create the main GUI Surface */
    fgui->MainScreen = SDL_CreateRGBSurface(SDL_HWSURFACE ,video_screen->w, video_screen->h, 32,rmask, gmask, bmask, amask);
    if(fgui->MainScreen == NULL) {
        printf("CreateRGBSurface failed: %s\n", SDL_GetError());
        return ( FALSE );
    }	
	
	fgui->MainScreen_FullScreen = SDL_CreateRGBSurface(SDL_HWSURFACE ,640, 480, 32,rmask, gmask, bmask, amask);
    if(fgui->MainScreen_FullScreen == NULL) {
        printf("CreateRGBSurface failed: %s\n", SDL_GetError());
        return ( FALSE );
    }	
	
	
	/*Load the Font 1 for the Widget Title Bar */
	fgui->Font1 = LoadFont("data/font_1.bmp");
	if (fgui->Font1 == NULL) {
		printf( "Couldnt load Font1\n");
		exit(1);
    }
	
	/*Load the Font 2 for the Widget Text */	
	fgui->Font2 = LoadFont("data/font_2.bmp");
	if (fgui->Font2 == NULL) {
		printf( "Couldnt load Font2\n");
		exit(1);
    }
	
	
	
	fgui->nbWidget 		= 0;					//At start , no widget				
	fgui->guiStyle		= muGUI_STYLE_DEFAULT;
	
	
	/*Initialisation of Default Functions Pointer*/
	fgui->run 			= muGUI_MainLoop;		//for the main loop
	fgui->clearScreen 	= muGUI_ClearScreen;	//for clearing screen
	fgui->updateStyle	= muGUI_UpdateStyle;	//for clearing screen	
	
	
	fgui->addWidget 	= muGUI_AddWidget;		//for adding Widget
	fgui->destroy 		= muGUI_Destroy;		//for destroying all the Widget and the GUI
	fgui->IdleProc 		= muGUI_defaultIdleProc;//for Idle Functions	
	
	/* GUI STATUS*/
	fgui->status 		= muGUI_STATUS_RUNNING;	
	
	
	/*Clear the GUI*/
	fgui->clearScreen(fgui);
	
	
	return ( TRUE );
}


/*
muGUI Destroy Functions
*/
void muGUI_Destroy (  void * this ) {
	int boucle;

	muGUI * zGui;
	zGui = (muGUI*) this;			//Get the muGUI handle
	
	printf("\nDestroy muGUI\n");		
	fflush(stdout);
	
	/* Free All Widgets */
	for ( boucle = 0 ; boucle < zGui->nbWidget ; boucle++) 
		 zGui->myWid[ boucle ].freeMem( &zGui->myWid[ boucle ] );
	
	/* Free All fonts */
 	FreeFont(zGui->Font1);
    FreeFont(zGui->Font2);
	
	/*Free muGUI Surface*/
	SDL_FreeSurface(zGui->MainScreen);
	
	printf("Destroy muGUI...Done\n");
}

/*
Default muGUI ClearScreen
Maybe show an image in next version in the background...
*/
void muGUI_ClearScreen (  void * this ) {
	
	SDL_Color rgb;
	Uint32 color;
 	
	muGUI * zGui;
	zGui = (muGUI*) this;		//Get the GUI handle

	/*Background Color*/
	rgb.r = 0x00;
	rgb.g = 0x69;	
	rgb.b = 0x99;	
	color = SDL_MapRGB(zGui->MainScreen->format, rgb.r, rgb.g, rgb.b);
 
 	/*Fill the background with the colo*/
   	SDL_FillRect(zGui->MainScreen, NULL, color);
	
	
	
	
}


/*
*/
void muGUI_UpdateStyle (  void * this ) {
	
//	SDL_Color rgb;
//	Uint32 color;
 	
	muGUI * zGui;
	zGui = (muGUI*) this;		//Get the GUI handle
	
	

	
	switch ( zGui->guiStyle ) {
		
		case muGUI_STYLE_DEFAULT : {
			
			SDL_SetVideoMode(800, 600, muGUI_MainScreenBPP, SDL_HWSURFACE | SDL_DOUBLEBUF | SDL_HWPALETTE );
			video_screen = SDL_GetVideoSurface ();		//Get the main screen Surface
			
			
			zGui->myWid[ nSX2_WID_VIDEO_OUTPUT ].external_draw = muGUIwidget_external_draw;
			zGui->myWid[ nSX2_WID_VIDEO_OUTPUT ].internal_draw = muGUIwidget_image_internal_draw;
			zGui->myWid[ nSX2_WID_VIDEO_OUTPUT ].x 		= 0 ;
			zGui->myWid[ nSX2_WID_VIDEO_OUTPUT ].y 		= 0 ;
			zGui->myWid[ nSX2_WID_VIDEO_OUTPUT ].Status =	MUGUI_WID_STATUS_SHOW;
						
			
			zGui->myWid[ nSX2_WID_EMU_OUTPUT ].x 		= 0;
			zGui->myWid[ nSX2_WID_EMU_OUTPUT ].y 		= 410;
			zGui->myWid[ nSX2_WID_EMU_OUTPUT ].Status 	= MUGUI_WID_STATUS_SHOW;
			

			zGui->myWid[ nSX2_WID_OPCODE_OUTPUT ].x 		= zGui->MainScreen->w + 1;
			zGui->myWid[ nSX2_WID_OPCODE_OUTPUT ].Status 	= MUGUI_WID_STATUS_HIDE;
			zGui->myWid[ nSX2_WID_OPCODE_OUTPUT ].h 		= 22;
			
			zGui->myWid[ nSX2_WID_REGISTER_OUTPUT ].x 		= zGui->MainScreen->w + 1;
			zGui->myWid[ nSX2_WID_REGISTER_OUTPUT ].Status 	= MUGUI_WID_STATUS_HIDE;
			zGui->myWid[ nSX2_WID_REGISTER_OUTPUT ].h 		= 22;

			zGui->myWid[ nSX2_WID_COP1REGISTER_OUTPUT ].x 		= zGui->MainScreen->w + 1;
			zGui->myWid[ nSX2_WID_COP1REGISTER_OUTPUT ].Status 	= MUGUI_WID_STATUS_HIDE;
			zGui->myWid[ nSX2_WID_COP1REGISTER_OUTPUT ].h 		= 22;		
		
			zGui->myWid[ nSX2_WID_VIDEO_OUTPUT ].moved = TRUE;
		
			break;	
		}
		case muGUI_STYLE_DEBUG : {

			SDL_SetVideoMode(800, 600, muGUI_MainScreenBPP, SDL_HWSURFACE | SDL_DOUBLEBUF | SDL_HWPALETTE );
			video_screen = SDL_GetVideoSurface ();		//Get the main screen Surface
	

			zGui->myWid[ nSX2_WID_VIDEO_OUTPUT ].external_draw = muGUIwidget_external_draw;
			zGui->myWid[ nSX2_WID_VIDEO_OUTPUT ].internal_draw = muGUIwidget_image_internal_draw;
			zGui->myWid[ nSX2_WID_VIDEO_OUTPUT ].x 		= 0 ;
			zGui->myWid[ nSX2_WID_VIDEO_OUTPUT ].y 		= 0 ;
			zGui->myWid[ nSX2_WID_VIDEO_OUTPUT ].Status =	MUGUI_WID_STATUS_HIDE;
			zGui->myWid[ nSX2_WID_VIDEO_OUTPUT ].h 		= 22;			
			
			zGui->myWid[ nSX2_WID_EMU_OUTPUT ].x 		= 0;
			zGui->myWid[ nSX2_WID_EMU_OUTPUT ].y 		= zGui->MainScreen->h - zGui->myWid[ nSX2_WID_EMU_OUTPUT ].h;
			zGui->myWid[ nSX2_WID_EMU_OUTPUT ].Status 	= MUGUI_WID_STATUS_SHOW;
			

			zGui->myWid[ nSX2_WID_OPCODE_OUTPUT ].x 		= zGui->myWid[ nSX2_WID_EMU_OUTPUT ].w;
			zGui->myWid[ nSX2_WID_OPCODE_OUTPUT ].y 		= zGui->MainScreen->h - zGui->myWid[ nSX2_WID_EMU_OUTPUT ].h;;
			zGui->myWid[ nSX2_WID_OPCODE_OUTPUT ].Status 	= MUGUI_WID_STATUS_SHOW;
			
			
			zGui->myWid[ nSX2_WID_REGISTER_OUTPUT ].x 		= 0;
			zGui->myWid[ nSX2_WID_REGISTER_OUTPUT ].y 		= zGui->myWid[ nSX2_WID_VIDEO_OUTPUT ].h;
			zGui->myWid[ nSX2_WID_REGISTER_OUTPUT ].Status 	= MUGUI_WID_STATUS_SHOW;
			

			zGui->myWid[ nSX2_WID_COP1REGISTER_OUTPUT ].x 		= zGui->myWid[ nSX2_WID_REGISTER_OUTPUT ].x + zGui->myWid[ nSX2_WID_REGISTER_OUTPUT ].w;
			zGui->myWid[ nSX2_WID_COP1REGISTER_OUTPUT ].y 		= zGui->myWid[ nSX2_WID_REGISTER_OUTPUT ].y;
			zGui->myWid[ nSX2_WID_COP1REGISTER_OUTPUT ].Status 	= MUGUI_WID_STATUS_SHOW;
					
		
			zGui->myWid[ nSX2_WID_VIDEO_OUTPUT ].moved = TRUE;

			
			
			
			break;	
		}
		case muGUI_STYLE_FULLSCREEN : {
			
			SDL_SetVideoMode(640, 480, muGUI_MainScreenBPP, SDL_HWSURFACE | SDL_DOUBLEBUF | SDL_HWPALETTE /* | SDL_FULLSCREEN */);
			video_screen = SDL_GetVideoSurface ();		//Get the main screen Surface
			
			zGui->myWid[ nSX2_WID_VIDEO_OUTPUT ].external_draw = muGUIwidget_null_external_draw;
			zGui->myWid[ nSX2_WID_VIDEO_OUTPUT ].internal_draw = muGUIwidget_image_internal_draw_fullscreen;
			zGui->myWid[ nSX2_WID_VIDEO_OUTPUT ].x 		= ( zGui->MainScreen->w - zGui->myWid[ nSX2_WID_VIDEO_OUTPUT ].w ) / 2 ;
			zGui->myWid[ nSX2_WID_VIDEO_OUTPUT ].y 		= ( zGui->MainScreen->h - zGui->myWid[ nSX2_WID_VIDEO_OUTPUT ].previous_h ) / 2 ;
			zGui->myWid[ nSX2_WID_VIDEO_OUTPUT ].Status =	MUGUI_WID_STATUS_SHOW;
						
			
			zGui->myWid[ nSX2_WID_EMU_OUTPUT ].x 		= zGui->MainScreen->w + 1;
			zGui->myWid[ nSX2_WID_EMU_OUTPUT ].Status 	= MUGUI_WID_STATUS_HIDE;
			zGui->myWid[ nSX2_WID_EMU_OUTPUT ].h 		= 22;

			zGui->myWid[ nSX2_WID_OPCODE_OUTPUT ].x 		= zGui->MainScreen->w + 1;
			zGui->myWid[ nSX2_WID_OPCODE_OUTPUT ].Status 	= MUGUI_WID_STATUS_HIDE;
			zGui->myWid[ nSX2_WID_OPCODE_OUTPUT ].h 		= 22;
			
			zGui->myWid[ nSX2_WID_REGISTER_OUTPUT ].x 		= zGui->MainScreen->w + 1;
			zGui->myWid[ nSX2_WID_REGISTER_OUTPUT ].Status 	= MUGUI_WID_STATUS_HIDE;
			zGui->myWid[ nSX2_WID_REGISTER_OUTPUT ].h 		= 22;

			zGui->myWid[ nSX2_WID_COP1REGISTER_OUTPUT ].x 		= zGui->MainScreen->w + 1;
			zGui->myWid[ nSX2_WID_COP1REGISTER_OUTPUT ].Status 	= MUGUI_WID_STATUS_HIDE;
			zGui->myWid[ nSX2_WID_COP1REGISTER_OUTPUT ].h 		= 22;		
		
			zGui->myWid[ nSX2_WID_VIDEO_OUTPUT ].moved = TRUE;
		
			break;	
		}
	
	}
	
		
}




/*
Default Idle Proc
Do Nothing but MUST exist
*/
void muGUI_defaultIdleProc ( void ) { }


/*
muGUI Main Loop
Handle the Refresh of the Widget

WARNING : 
-
-The Widget drawing order is the Widget creation order

*/


SDL_Surface * video_screen;

void muGUI_MainLoop (  void * this ) {
	
	unsigned int boucle;
	SDL_Surface * video_screen;
	SDL_Rect rect;
	int result;
   	SDL_Event event;
   	
   	int NeedGlobalRefresh;	
   	int NeedPartialRefresh;
   	
  		
	muGUI * zGui;
	zGui = (muGUI*) this;						//Get the muGUI handle

	video_screen = SDL_GetVideoSurface ();		//Get the main screen Surface
	
	
	/*
	While the GUI is running....
	*/
	while ( zGui->status == muGUI_STATUS_RUNNING ) {
	
	
	
		/*Execute the idle fucntion*/
		zGui->IdleProc();

		/*I don't need a Refresh*/
		NeedGlobalRefresh 	= FALSE;
		NeedPartialRefresh 	= FALSE;
	
		/* Check if widgets need to be refresh*/
		for ( boucle = 0 ; boucle < zGui->nbWidget ; boucle++) 	{
			
			if ( zGui->myWid[ boucle ].NeedRefresh == TRUE ) NeedPartialRefresh = TRUE ;
					
			if ( zGui->myWid[ boucle ].moved == TRUE ) {
	
					NeedGlobalRefresh = TRUE ;
					zGui->myWid[ boucle ].moved = FALSE;
					
				}
		}
		
			/*if a partial  refresh is needed*/		
		if ( NeedPartialRefresh == TRUE  && NeedGlobalRefresh == FALSE) {
			
			
			/*For each widget...*/
			for ( boucle = 0 ; boucle < zGui->nbWidget ; boucle++) {
				
				if ( zGui->myWid[ boucle ].NeedRefresh == TRUE ) {
					
										
					/*...draw the internal ( means : image or text ) if the widget is not Hidden*/
					if ( zGui->myWid[ boucle ].Status == MUGUI_WID_STATUS_SHOW ) 
						zGui->myWid[ boucle ].internal_draw( &(zGui->myWid[ boucle ]) , zGui );
				
					/* the Widget don't need to be refresh */
					zGui->myWid[ boucle ].NeedRefresh = FALSE;
					
					
					rect.x = zGui->myWid[ boucle ].x;	//Set the Rectangle
				  	rect.y = zGui->myWid[ boucle ].y;
				  	rect.w = zGui->myWid[ boucle ].w;
				  	rect.h = zGui->myWid[ boucle ].h;
					
					
					if ( zGui->guiStyle != muGUI_STYLE_FULLSCREEN ) {
					
						/* Copy the only the widget part to the screen */
						result = SDL_BlitSurface(zGui->MainScreen , &rect, video_screen, &rect);
						SDL_UpdateRect( video_screen,0,0,0,0 );
					}
					else {
						/*else Copy the Entire Screen*/
						result = SDL_BlitSurface(zGui->MainScreen_FullScreen , NULL, video_screen, NULL);
						SDL_Flip( video_screen );
						
					}
					
				
					
					
				}

				
			}
		
		}
		
		
		
		/*if a global refresh is needed*/		
		if ( NeedGlobalRefresh == TRUE ) {
			
			/*Clear Screen*/
			zGui->clearScreen(zGui);
			
			/*For each widget...*/
			for ( boucle = 0 ; boucle < zGui->nbWidget ; boucle++) {
				
				/*...draw the external ( means : Title Bar... ) */
				zGui->myWid[ boucle ].external_draw( &(zGui->myWid[ boucle ]) , zGui );
				
				/*...draw the internal ( means : image or text ) if the widget is not Hidden*/
				if ( zGui->myWid[ boucle ].Status == MUGUI_WID_STATUS_SHOW ) 
					zGui->myWid[ boucle ].internal_draw( &(zGui->myWid[ boucle ]) , zGui );
				
				
				
			}

			

			/* Copy the GUI Surface to the screen */
			result = SDL_BlitSurface(zGui->MainScreen , NULL, video_screen, NULL);
			SDL_UpdateRect( video_screen,0,0,0,0 );
		
			
		}
		
		/*Check and Execute Event*/
		SDL_PollEvent(&event);

	}		
		

}


/*
Add a Widget to the GUI

WARNING :
Due to STATIC memory Allocation , take care of initilize all the Widget Struc Element
*/
void muGUI_AddWidget ( void * this , muGUIwidget * fWid ) {
	
	muGUI * zGui;
	zGui = (muGUI*) this;		//Get the GUI handle

	/* if the widget is created */
	if ( fWid != NULL ) {		

		/*Copy the Widget...*/
		zGui->myWid[ zGui->nbWidget ].ID 			= zGui->nbWidget ;		//... ID
		zGui->myWid[ zGui->nbWidget ].Type 			= fWid->Type ;			//... Type 
		zGui->myWid[ zGui->nbWidget ].x 			= fWid->x ;				//... x position
		zGui->myWid[ zGui->nbWidget ].y 			= fWid->y ;				//... y position
		zGui->myWid[ zGui->nbWidget ].w 			= fWid->w ;				//... wi
		zGui->myWid[ zGui->nbWidget ].previous_h	= fWid->h ;				//... wi
		zGui->myWid[ zGui->nbWidget ].h 			= fWid->h ;				//... length
		zGui->myWid[ zGui->nbWidget ].NeedRefresh 	= fWid->NeedRefresh ;	//... refresh status
		zGui->myWid[ zGui->nbWidget ].moved 		= fWid->moved ;			//... movement status
		zGui->myWid[ zGui->nbWidget ].Status 		= fWid->Status;			//... status
		zGui->myWid[ zGui->nbWidget ].external_draw = fWid->external_draw ;	//... external function
		zGui->myWid[ zGui->nbWidget ].internal_draw = fWid->internal_draw ;	//... internal function
		zGui->myWid[ zGui->nbWidget ].isClick 		= fWid->isClick ;		//... Is click function
		zGui->myWid[ zGui->nbWidget ].freeMem 		= fWid->freeMem ;		//... destroy function	
		strcpy( zGui->myWid[ zGui->nbWidget ].Title, fWid->Title );			//... Title
	
		/*..for the Image Widget..*/
		zGui->myWid[ zGui->nbWidget ].image 		= fWid->image ;			//... SDL surface
		zGui->myWid[ zGui->nbWidget ].addImage 		= fWid->addImage ;		//... add image function
		
		/*..for the Text Widget..*/
		zGui->myWid[ zGui->nbWidget ].TextBuffer 	= fWid->TextBuffer ;	//... chained text buffer
		zGui->myWid[ zGui->nbWidget ].addText  		= fWid->addText;		//... add Text function
		zGui->myWid[ zGui->nbWidget ].maxLines  	= fWid->maxLines;		//... max line to display
		zGui->myWid[ zGui->nbWidget ].displayAtLine = fWid->displayAtLine;	//... start display line
	
	
		/*Incremente the number of Widget*/
		zGui->nbWidget = zGui->nbWidget + 1;
	}

	
	
}




/*Just print text to various Widget */

void PrintToEmuOutput (  char * ftext ) {
	
	nSX2GUI.myWid[nSX2_WID_EMU_OUTPUT].addText(&nSX2GUI.myWid[nSX2_WID_EMU_OUTPUT],ftext);
}

void PrintToOpcodeOutput (  char * ftext ) {

	nSX2GUI.myWid[nSX2_WID_OPCODE_OUTPUT].addText(&nSX2GUI.myWid[nSX2_WID_OPCODE_OUTPUT],ftext);
	//printf("%s\n",ftext);
}

void PrintToRegistersOutput (  char * ftext ) {
	
	nSX2GUI.myWid[nSX2_WID_REGISTER_OUTPUT].addText(&nSX2GUI.myWid[nSX2_WID_REGISTER_OUTPUT],ftext);

} 

void PrintToCOP1RegistersOutput (  char * ftext ) {
	
	nSX2GUI.myWid[nSX2_WID_COP1REGISTER_OUTPUT].addText(&nSX2GUI.myWid[nSX2_WID_COP1REGISTER_OUTPUT],ftext);

} 