/**
 * nSX2 - an OpenSource PS2 Emulator
 * Copyright (C) 2002 Muad
 *
 * homepage: http://nsx2.emulation64.com
 * email address: atreides.muaddib@caramail.com
 * 
 * If you want to contribute to the project please contact
 * me first (maybe someone is already making what you are
 * planning to do).
 *
 *
 * This program is free software; you can redistribute it and/
 * or modify it under the terms of the GNU General Public Li-
 * cence as published by the Free Software Foundation; either
 * version 2 of the Licence, or any later version.
 *
 * This program is distributed in the hope that it will be use-
 * ful, but WITHOUT ANY WARRANTY; without even the implied war-
 * ranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public Licence for more details.
 *
 * You should have received a copy of the GNU General Public
 * Licence along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
**/


/********************************************************************************************
							GENERAL EMULATOR DEFINITION
*********************************************************************************************/

#include <SDL/SDL.h>



#define nSX2_VERSION	0
#define nSX2_BUILD		4
#define sSX2_BUILD_DATE "20021004"

#define	nSX2ConfigFile	"./nsx2.ini"

/*
Use the GS pluggin
*/
#define GSdefs





/********************************************************************************************
							TYPE DEFINITION
*********************************************************************************************/
#define TRUE 	1
#define FALSE 	0

typedef unsigned long long	_u64;
typedef unsigned long		_u32;
typedef unsigned short		_u16;
typedef unsigned char		_u8;

typedef	long long			_s64;
typedef	long				_s32;
typedef	short				_s16;
typedef	char				_s8;

/********************************************************************************************
							EMULATOR DEFINES
*********************************************************************************************/

#define CPU_REG_R0	0
#define CPU_REG_AT	1
#define CPU_REG_V0	2
#define CPU_REG_V1	3
#define CPU_REG_A0	4
#define CPU_REG_A1	5
#define CPU_REG_A2	6
#define CPU_REG_A3	7
#define CPU_REG_T0	8
#define CPU_REG_T1	9
#define CPU_REG_T2	10
#define CPU_REG_T3	11
#define CPU_REG_T4	12
#define CPU_REG_T5	13
#define CPU_REG_T6	14
#define CPU_REG_T7	15
#define CPU_REG_S0	16
#define CPU_REG_S1	17
#define CPU_REG_S2	18
#define CPU_REG_S3	19
#define CPU_REG_S4	20
#define CPU_REG_S5	21
#define CPU_REG_S6	22
#define CPU_REG_S7	23
#define CPU_REG_T8	24
#define CPU_REG_T9	25
#define CPU_REG_K0	26
#define CPU_REG_K1	27
#define CPU_REG_GP	28
#define CPU_REG_SP	29
#define CPU_REG_S8	30
#define CPU_REG_RA	31
#define CPU_REG_HI	32
#define CPU_REG_LO	33

#define SIGN_BIT_MASK       0x80000000
#define SIGN_BIT_MASK_64    0x8000000000000000

/*
COP 1 Control Register
*/
#define CP1_FIR		0	//Floating-Point Implementation Register (FIR, CP1 Control Register 0)
#define CP1_FCCR	25	//Floating-Point Condition Codes Register (FCCR, CP1 Control Register 25)
#define CP1_FEXR	26	//Floating-Point Exceptions Register (FEXR, CP1 Control Register 26)
#define CP1_FENR	28	//Floating-Point Enables Register (FENR, CP1 Control Register 28)
#define CP1_FCSR	31	//Floating-Point Control and Status Register (FCSR, CP1 Control Register 31)

#define CP1_FCSR_C_MASK     0x00800000
#define CP1_FCSR_C_NOT_MASK 0xFF7FFFFF

/********************************************************************************************
							STRUCTURES DEFINITIONS
*********************************************************************************************/


/*
128 bits structure
*/
typedef struct __u128 {
	_u64 lo;
	_u64 hi;
} _u128;


typedef struct _CpuOpcodePrefetch {
	
	void (*fct_opcode)(void) ;
	_u32	opcode;
		
	_u32	rt;
	_u32	rs;
	_u32	rd;
	_u32	base;
	_u32	sa;
	_u64	imm;
	_u64	offset;
	_u64	instr_index;
	_u32	sel;
	_u32	ft;
	_u32	fd;
	_u32	fs;
	_u32	cc;
	_u32	ccb;	
	
	
} CpuOpcodePrefetch;




/*
Main CPU R5900
*/
typedef	struct _Mips_CPU {
	_u32		opcode;					//current opcode
	_u8			halt;					//if 1 stop the emu
	
	_u128 		GPR[34];				//General Registers
	
	_u64		CPR0[32][4];			//Cop0	Control Register

	_u128		FPR1[32];				//COP1	FloatingPoint Register for the CPU
    float		FPR1_float[32];			//COP1	FloatingPoint Register for the EMU	
    float		*FPR1_p_float[32];		//COP1	Pointer Float for conversion
	_u64		*FPR1_p_u64[32];		//COP1	Pointer u64 for conversion
	
	
	_u64		CP1[32];				//COP1 Control Register
	
	_u64		Nb_instructions;		//Number Of Instruction Exececuted
	_u64		PC;						//Program Counter
	_u32 		EntryPointAdd;			//Start the emu at
	
	
	/* Interpreter V2 */
	
	_u32		NbOfExecutableSection;
	_u32		OpcodeStartAdd[10];
	_u32		OpcodeEndAdd[10];
	
	_u32		IndexListPrefetchedOpcode;
	CpuOpcodePrefetch * CurrentOpcode;
	CpuOpcodePrefetch ** ListPrefetchedOpcode;	
	
} Mips_CPU;


/*
DMA Channel
*/
typedef struct _DmaChannel {

	_u64		ctrl;			//Channel Control register
	_u64		memAdd;			//Memory address
	_u64		size;			//Quadra-word counter
	_u64		tag;			//Tag address
	_u64		asr0;			//Assembler Reserved 0
	_u64		asr1;			//Assembler Reserved 1
	_u64		saddr;			//???

} DmaChannel;

/*
DMA HANDLER
*/

typedef struct	_DmaHandler {
	
	_u8			Enable;			//Enable or Disable the Handler
	_u64		Add;			//Adress of the Handler
	_u32        EndAdd;
    _u64		ReturnAdd;		//Return Add
	_u8			Running;		//???
		
} DmaHandler;

/*
SIF
*/

typedef struct _SifHw {
	
	_u8		Enable;			//Enable or disable the sif
	_u64	DChain;			//Adress of the chain

	
} SifHw;

typedef struct _Bios_Sema {
    
    _u64    MemAddress;

    _u16    current_count;
    _u16    max_count;
    _u16    initial_count;
    
} Bios_Sema;



/*
Internal Emulator BIOS
*/

#define		BIOS_MAX_SEMAPHORE		32
#define		BIOS_MAX_SIF_DCHAIN		256
#define		BIOS_MAX_DMA_CHANNEL	10

typedef	struct _Bios_Hw {

	_u64		GsCrt;
	
	/*
	Detecting Acces to DMA Area Memory
	*/
	_u8		WriteToDmaArea;
	_u64	DmaAreaWriteAt;
	_u64	DmaAreaWriteVal;
	
	_u8		ReadToDmaArea;
	_u64	DmaAreaReadAt;
	_u64	DmaAreaReadVal;	
	
	_u16       MustExecHandler;
	_u128      SaveReg[34];
	
	/* DMA CHANNEL */
	DmaChannel DmaC[BIOS_MAX_DMA_CHANNEL];
	
	/* DMA HANDLER */
	DmaHandler	DmaH[BIOS_MAX_DMA_CHANNEL];
	
	
	/* SEMAPHORE STUFF */
	_u8 	NbSema;
	Bios_Sema	Sema[BIOS_MAX_SEMAPHORE];
	
	
	/*SIF*/
	SifHw 	Sif[2];
	
	/* VERY BAD WAY*/
	_u64	NeedUpdateScreen;

} Bios_Hw;





/********************************************************************************************
							EXTERN GLOBALS VARIABLES
*********************************************************************************************/

extern FILE * ElfFile;		//Current Elf File
extern long ElfFileSize;	//Size of the Elf File
extern FILE * BiosFile;		//Current BIOS File


extern Mips_CPU MipsCpu;	//Main CPU
extern Bios_Hw  Ps2Bios;	//Internal BIOS



extern _u8	*MipsMemory;			//Main Memory 32 Mbits
extern _u8	*SpecialAreaMemory;		//Special Memory ( PS2 Control Register  )
extern _u8	*GSAreaMemory;			//GS control register
extern _u8	*BiosMemory;			//Bios Memory

extern char ElfToExecute[256];		//Elf File to Execute


extern _u16		muGUI_MainScreenW;
extern _u16		muGUI_MainScreenH;
extern _u16		muGUI_MainScreenBPP;
extern _u16		muGUI_MainScreenStretch;


/********************************************************************************************
							MACROS DEFINITIONS
*********************************************************************************************/
//From Hacktarux
#define sign_extended(a) if (a & 0x0000000080000000LL) a |= 0xFFFFFFFF00000000LL; else a &= 0xFFFFFFFF
#define sign_extendedb(a) if (a & 0x80) a |= 0xFFFFFFFFFFFFFF00LL; else a &= 0xFF
#define sign_extendedh(a) if (a & 0x8000) a |= 0xFFFFFFFFFFFF0000LL; else a &= 0xFFFF


//This macro decode the memory and Fill the opcode field of the cpu
#define MipsFillOpcode(A) MipsCpu.opcode=*( (_u64 *) (( _u8 *)( MipsMemory + A ) ))


//These macro Extract bits from opcode and fill appropriate var


    #define Extract_rt() 		rt=(MipsCpu.opcode&0x1F0000)>>16
    #define Extract_rs() 		rs=(MipsCpu.opcode&0x3E00000)>>21
    #define Extract_base() 		base=(MipsCpu.opcode&0x3E00000)>>21

    #define Extract_imm() 		imm=(MipsCpu.opcode&0xFFFF)
    #define Extract_offset() 	offset=(MipsCpu.opcode&0xFFFF)

    #define Extract_rd() 		rd=(MipsCpu.opcode&0xF800)>>11

    #define Extract_instr_index() instr_index=(MipsCpu.opcode&0x3FFFFFF)
    #define Extract_sa() 		sa=(MipsCpu.opcode&0x7C0)>>6
    #define Extract_sel() 		sa=(MipsCpu.opcode&0x7)

    #define Extract_ft() 		ft=(MipsCpu.opcode&0x1F0000)>>16
    #define Extract_fd() 		fd=(MipsCpu.opcode&0x7C0)>>6
    #define Extract_fs() 		fs=(MipsCpu.opcode&0xF800)>>11
    #define Extract_cc() 		cc=(MipsCpu.opcode&0x700)>>8
    #define Extract_ccb() 		ccb=(MipsCpu.opcode&0x1C0000)>>18







//This macro decode the memory and fill a var named B
#define ReadMemAt(B,A)  B==*( (_u64 *) (( _u8 *)( MipsMemory + A ) ))



    #define INCREMENT_PC() MipsCpu.PC = MipsCpu.PC + 4
   





/*******************************************************************************************/
void muGUI_nSX2Proc ( void ) ;
void muGUI_nSX2InitProc ( void ) ;
void muGUI_nSX2EndingProc ( void ) ;


/********************************************************************************************
							ELF FILE DECLARATIONS
*********************************************************************************************/
long 	FileSizeOf			( FILE *fp );
_u8 	ElfOpenFile			( char * fFileName );
void 	ElfCloseFile		(void);
_u8 	ElfLoadHeader		(void);
void 	ElfPrintInfo		(void);
void 	ElfPrintSectionInfo	( _u8 fIndex );

void 	BiosCloseFile		(void);
_u8 	BiosOpenFile		( char * fFileName );



/********************************************************************************************
							MEMORY DECLARATIONS
*********************************************************************************************/
#define DATA_QUAD		1		//128 bits	
#define DATA_DW			2		//64 bits	
#define DATA_DWH		32		//64 bits
#define DATA_W			4		//32 bits
#define DATA_HW			8		//16 bits
#define DATA_B			16		//8 bits

_u8 MipsMemoryAlloc				(void);
_u8 MipsSpecialAreaMemoryAlloc	(void);
_u8 MipsGSAreaMemoryAlloc		(void);
_u8 MipsBiosMemoryAlloc			(void);

void 	FreeAllMemory			(void);
void 	CopyProgramToMemory		(void);
void 	CopySectionToMemory 	(void);
void 	CopyBiosToMemory 		(void);

void	MipsMemoryWrite 	( _u128 fVal,	_u8 fDataType,	_u64 fBase, 	int fOffset );
void	MipsSpecialMemWrite	( _u8 * Mem,	_u128  fVal,	_u8 fDataType, 	_u64 fBase,	int fOffset );
void	MipsGSAreaMemWrite	( _u8 * Mem,	_u128  fVal,	_u8 fDataType, 	_u64 fBase,	int fOffset );

void	MipsMemoryWriteDW 	( _u8 * Mem,	_u64 fVal,		_u64 fAdd );
void	MipsMemoryWriteW  	( _u8 * Mem,	_u64 fVal,		_u64 fAdd );
void	MipsMemoryWriteHW  	( _u8 * Mem,	_u64 fVal,		_u64 fAdd );
void	MipsMemoryWriteB  	( _u8 * Mem,	_u64 fVal,		_u64 fAdd );

void	MipsMemoryRead 		( _u128 * fVal,	_u8 fDataType,	_u64 fBase,		int fOffset );
void	MipsSpecialMemRead	( _u8 * Mem,	_u128 * fVal,	_u8 fDataType,	_u64 fBase,	int fOffset );
void	MipsGSAreaMemRead	( _u8 * Mem,	_u128 * fVal,	_u8 fDataType,	_u64 fBase,	int fOffset );
void	MipsBiosAreaMemRead	( _u8 * Mem,	_u128 * fVal,	_u8 fDataType,	_u64 fBase,	int fOffset );

void 	MipsMemoryReadDQ 	( _u8 * Mem,  	_u128 * fVal , 	_u64 fAddr );
void	MipsMemoryReadDWH 	( _u8 * Mem,	_u128 * fVal,	_u64 PhysicalAdd );
void	MipsMemoryReadDW 	( _u8 * Mem,	_u128 * fVal,	_u64 PhysicalAdd );
void	MipsMemoryReadW 	( _u8 * Mem,	_u128 * fVal,	_u64 fAddr );
void	MipsMemoryReadHW 	( _u8 * Mem,	_u128 * fVal,	_u64 fAddr );
void	MipsMemoryReadB 	( _u8 * Mem,	_u128 * fVal,	_u64 fAddr );


/********************************************************************************************
							CONFIG FILE DECLARATIONS
*********************************************************************************************/
int 	nSX2LoadConfig 	(void);



/********************************************************************************************
							MAIN CPU R5900 DECLARATIONS
*********************************************************************************************/
void	MipsUpdateCpu 		(void);
void	MipsCpuPrefetchOpcode(void);

_u8		debug_op;
_u8		debug_gs;
_u8		debug_dma;
_u8		debug_print_syscall;

/********************************************************************************************
							GS MAIN DECLARATIONS
*********************************************************************************************/
void 	GSUpdate			(void);




/********************************************************************************************
							DMA DECLARATIONS
*********************************************************************************************/

#define DMA_CH0 	0		//Linked to VIF0
#define DMA_CH1 	1		//Linked to VIF1
#define DMA_CH2 	2		//Linked to GIF
#define DMA_CH3 	3		//Linked to IPU (FROM???)
#define DMA_CH4 	4		//Linked to IPU (TO???)
#define DMA_CH5 	5		//Linked to SIF0
#define DMA_CH6 	6		//Linked to SIF1
#define DMA_CH7 	7		//Linked to SIF2
#define DMA_CH8 	8		//Linked to SPR  (from SCRATCH PAD to RAM???)
#define DMA_CH9 	9		//Linked to SPR  (from RAM to SCRATCH PAD???)
#define DMA_CONTROL 10		


#define DMA_CTRL    0x1000E000          //DMAC Control
#define DMA_STAT    0x1000E010          //DMAC Status
#define DMA_PCR     0x1000E020          //DMAC Priority control
#define DMA_SQWC    0x1000E030          //DMAC skip quad word
#define DMA_RBSR    0x1000E040          //DMAC ring buffer size
#define DMA_RBOR    0x1000E050          //DMAC Ring buffer offset
#define DMA_STADR   0x1000E060          //DMA stall address




void	DmaUpdate			(void);
void 	DmaPrintChannelInfo ( _u8 fChannel );
