/**
 * nSX2 - an OpenSource PS2 Emulator
 * Copyright (C) 2002 Muad
 *
 * homepage: http://nsx2.emulation64.com
 * email address: atreides.muaddib@caramail.com
 * 
 * If you want to contribute to the project please contact
 * me first (maybe someone is already making what you are
 * planning to do).
 *
 *
 * This program is free software; you can redistribute it and/
 * or modify it under the terms of the GNU General Public Li-
 * cence as published by the Free Software Foundation; either
 * version 2 of the Licence, or any later version.
 *
 * This program is distributed in the hope that it will be use-
 * ful, but WITHOUT ANY WARRANTY; without even the implied war-
 * ranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public Licence for more details.
 *
 * You should have received a copy of the GNU General Public
 * Licence along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
**/
#include <stdio.h>
#include <stdlib.h>

#include <float.h>
#include <math.h>

#include "nSX2.h"
#include "r5900_main.h"
#include "r5900_main_table.h"
 
#include "muGUI/muGUI.h"
 
#define		CURRENT_RT				MipsCpu.CurrentOpcode->rt
#define		CURRENT_RS				MipsCpu.CurrentOpcode->rs
#define		CURRENT_RD				MipsCpu.CurrentOpcode->rd
#define		CURRENT_BASE			MipsCpu.CurrentOpcode->base
#define		CURRENT_SA				MipsCpu.CurrentOpcode->sa
#define		CURRENT_IMM				MipsCpu.CurrentOpcode->imm
#define		CURRENT_OFFSET			MipsCpu.CurrentOpcode->offset
#define		CURRENT_INSTR_INDEX		MipsCpu.CurrentOpcode->instr_index
#define		CURRENT_SEL				MipsCpu.CurrentOpcode->sel
#define		CURRENT_FT				MipsCpu.CurrentOpcode->ft
#define		CURRENT_FD				MipsCpu.CurrentOpcode->fd
#define		CURRENT_FS				MipsCpu.CurrentOpcode->fs
#define		CURRENT_CC				MipsCpu.CurrentOpcode->cc
#define		CURRENT_CCB				MipsCpu.CurrentOpcode->ccb
 



char debugTempText[100];
char debugTempOpcodeText[100];


/*
Macro to extract the field of the Opcode
*/
#define PrefetchExtractrt() 		MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->rt=((MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->opcode)&0x1F0000)>>16
#define PrefetchExtractrs() 		MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->rs=((MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->opcode)&0x3E00000)>>21
#define PrefetchExtractbase() 		MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->base=((MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->opcode)&0x3E00000)>>21

#define PrefetchExtractimm() 		MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->imm=((MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->opcode)&0xFFFF)
#define PrefetchExtractoffset() 	MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset=((MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->opcode)&0xFFFF)

#define PrefetchExtractrd() 		MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->rd=((MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->opcode)&0xF800)>>11

#define PrefetchExtractinstr_index() MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->instr_index=((MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->opcode)&0x3FFFFFF)
#define PrefetchExtractsa() 		MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->sa=((MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->opcode)&0x7C0)>>6
#define PrefetchExtractsel() 		MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->sa=((MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->opcode)&0x7)

#define PrefetchExtractft() 		MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->ft=((MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->opcode)&0x1F0000)>>16
#define PrefetchExtractfd() 		MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->fd=((MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->opcode)&0x7C0)>>6
#define PrefetchExtractfs() 		MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->fs=((MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->opcode)&0xF800)>>11
#define PrefetchExtractcc() 		MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->cc=((MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->opcode)&0x700)>>8
#define PrefetchExtractccb() 		MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->ccb=((MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->opcode)&0x1C0000)>>18






/*
Before Executing the file
I read all the opcode and I decode them
So during Execution I don't need to do it

*/
void	MipsCpuPrefetchOpcode( void ) {

	_u32	MemorySize;
	_u32	boucle;
	_u32	ReadAdd;
	
	/*Various Initialisation*/
	MemorySize =   (MipsCpu.OpcodeEndAdd[0] - MipsCpu.OpcodeStartAdd[0]) >> 2  ;	//Compute the size of the buffer
	ReadAdd = MipsCpu.OpcodeStartAdd[0] ;											//Must read at
	MipsCpu.IndexListPrefetchedOpcode = 0;											//Opcode already prefetched
	
	printf("Prefetch:Alloc Mem/Copy/Prefetch Opcodes \n");
	
	//Allocation memory for the pointer
	MipsCpu.ListPrefetchedOpcode = malloc ( ( sizeof (CpuOpcodePrefetch * )) * MemorySize );
	
	
	
	for ( boucle = 0 ; boucle < MemorySize ; boucle++ ) {
		
		//Allocation memory for the structure prefect Opcode
		MipsCpu.ListPrefetchedOpcode[boucle] = malloc ( sizeof ( CpuOpcodePrefetch ) );

		//Copy the opcode to the structure	
		MipsCpu.ListPrefetchedOpcode[boucle]->opcode = *( (_u64 *) (( _u8 *)( MipsMemory + ReadAdd ) ));		//This is the Macro : MipsFillOpcode
		ReadAdd = ReadAdd + 4;


		//Prefetch the opcode
		if ( MipsCpu.ListPrefetchedOpcode[boucle]->opcode != 0 ) pnormal_opc[((MipsCpu.ListPrefetchedOpcode[boucle]->opcode >> 26) & 0x3F)]();	//Execute the current opcode
		else	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = NOP;

		
		MipsCpu.IndexListPrefetchedOpcode = MipsCpu.IndexListPrefetchedOpcode + 1;
		
		printf("\rDone:%3ld %%",((boucle*100)/MemorySize)+1);
		fflush(stdout);
	}
	 
	

	
	
	
}





 
/*
Print the the current opcode
*/
void PrintDebugOpcode() {
	//printf("==>%8lx",MipsCpu.opcode);
	
	switch ( ((MipsCpu.opcode >> 26) & 0x3F) ) {
	
		case 0 	: 	printf("r5900:dbg_opcode_special: %s \n", debug_special_opc[(MipsCpu.opcode  & 0x3F)]); break;
		case 1	: 	printf("r5900:dbg_opcode_regimm: %s \n", debug_regimm_opc[(MipsCpu.opcode  & 0x1F0000)>>16]); break;
		case 16 : 	printf("r5900:dbg_opcode_cop0: %s \n", debug_cop0_opc[(MipsCpu.opcode&0x3E00000)>>21]);
				  	printf("r5900:dbg_opcode_cop0c0: %s \n", debug_cop0c0_opc[(MipsCpu.opcode&0x3F)]);
					break;
		case 17 : {
					switch( (MipsCpu.opcode&0x3E00000)>>21) {
						
						case 8:  printf("r5900:debug_cop1_BC1_opc: %s \n",	debug_cop1_bc1_opc[(MipsCpu.opcode>>16)&0x3] );break;					
						case 16: printf("r5900:debug_cop1_S_opc: %s \n", 	debug_cop1_s_opc[MipsCpu.opcode&0x3F] 	);break;					
						case 20: printf("r5900:debug_cop1_W_opc: %s \n", 	debug_cop1_W_L_opc[MipsCpu.opcode&0x3F] );break;					

						default: printf("r5900:dbg_opcode_cop1: %s \n", debug_cop1_opc[(MipsCpu.opcode&0x3E00000)>>21]);break;					
					}
					
					break;
				}	
	
		case 28 : printf("r5900:dbg_opcode_special2: %s \n", debug_special2_opc[(MipsCpu.opcode  & 0x3F)] );
	
		default : printf("r5900:dbg_opcode: %s \n", debug_normal_opc[((MipsCpu.opcode >> 26) & 0x3F)]); break;
	
	}

}
 
/*
Print the CPU Registers
Only the main registers and only the low part
*/
void MipsCpuPrintRegtoOutput( void ) {
	_u8	i=0;
	
	for ( i = 0 ; i < 34 ; i ++ ) {
		sprintf(debugTempOpcodeText,"%s : %16.16llx",debug_r5900_reg[ i ], MipsCpu.GPR[i].lo );
		PrintToRegistersOutput ( debugTempOpcodeText );
	}
}


void MipsCpuFPR1PrintRegtoOutput( void ) {
	_u8	i;

	for ( i = 0 ; i < 32 ; i ++ ) {
		sprintf(debugTempOpcodeText,"%4s : %12.6f",debug_r5900_cop1_reg[ i ], *(MipsCpu.FPR1_p_float[ i ]) );
		PrintToCOP1RegistersOutput ( debugTempOpcodeText );
	}
}



/*
Main CPU functions
Original Version
*/

void MipsUpdateCpuOriginal ( ) {
		
	MipsCpu.Nb_instructions = MipsCpu.Nb_instructions + 1 ;			
	
										//Read the memory and store the val to MipsCpu.opcode
	
	//if the opcode is different of NOP
	if ( MipsCpu.opcode != 0 ) normal_opc[((MipsCpu.opcode >> 26) & 0x3F)]();	//Execute the current opcode

    //else the opcode is NOP
	else {															
		if ( debug_op == 1 ) sprintf(debugTempText,"%8.8llx: nop", MipsCpu.PC);				
	    INCREMENT_PC();
 	}
	
	if ( debug_op == 1 ) {
		PrintToOpcodeOutput ( debugTempText );
		MipsCpuPrintRegtoOutput();
		MipsCpuFPR1PrintRegtoOutput();
	}
	
}

/*
Main CPU functions
Prefetched Version
*/

void MipsUpdateCpu ( ) {
	
	_u32	Index;

	/*Find the index where the opcode was prefetched*/	
	Index = (( MipsCpu.PC - MipsCpu.OpcodeStartAdd[0] ) >> 2 ) ;
	MipsCpu.CurrentOpcode = MipsCpu.ListPrefetchedOpcode[ Index ];
	 
	/* check if the Opcode is correct
		Not sure the PS2 do not Self Modifiing Code
	*/
	MipsFillOpcode(MipsCpu.PC);
	if ( MipsCpu.opcode != MipsCpu.CurrentOpcode->opcode ) printf("Error : IndexPrefetch NOT CORRECT\n");
	
	
	/*Execute the Opcode*/
	MipsCpu.CurrentOpcode->fct_opcode();
	
	/*Update the Number of Instructions*/
	MipsCpu.Nb_instructions = MipsCpu.Nb_instructions + 1 ;		
	
	
	if ( debug_op == 1 ) {
		PrintToOpcodeOutput ( debugTempText );
		MipsCpuPrintRegtoOutput();
		MipsCpuFPR1PrintRegtoOutput();
	}
	
}




/*
Test if a value is SignExtended
*/
_u8 IsSignExtended( _u64 fval ) {

    _u64    temp;
    
    temp = fval & 0xFFFFFFFF00000000;
    if (    ( temp == 0 ) || ( temp == 0xFFFFFFFF00000000 ) ) return TRUE;
    else return FALSE;        
 
    
}

/*
Test if a value is Overflow
*/
_u8 IsNot32bitIntegerOverflow ( _u64  fval) {

       _u32    temp;
    
    temp = ( (~fval) + 1 );

    if ( temp == 0 ) return FALSE;
    else return TRUE;


    
}


/*
Not sure
*/
_u8 IsNot64bitIntegerOverflow ( _u64  fval) {

    _u64    temp;
    
    temp = ( (~fval) + 1 );

    if ( temp == 0 ) return FALSE;
    else return TRUE;


        
}

/*
Not Implemented function
*/
void    NI()		{	

    PrintDebugOpcode();
	MipsCpu.halt = 1;
}

void    pNI()		{	
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = NI;
}

/*
N OPeration function
*/
void    NOP() 		{ 	INCREMENT_PC(); }
  

/*****************************************************************************************************************
									CPU INSTRUCTION SET
*****************************************************************************************************************/


void 	SPECIAL()	{	special_opc[(MipsCpu.opcode  & 0x3F)]();	}
void 	pSPECIAL()	{	pspecial_opc[((MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->opcode)  & 0x3F)]();	}

void 	REGIMM()	{	regimm_opc[(MipsCpu.opcode  & 0x1F0000)>>16]();		}
void 	pREGIMM()	{	pregimm_opc[((MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->opcode)  & 0x1F0000)>>16]();		}


void    COP0()		{	cop0_opc[(MipsCpu.opcode&0x3E00000)>>21]();}
void    pCOP0()		{	pcop0_opc[((MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->opcode)&0x3E00000)>>21]();}

void    COP1()		{	cop1_opc[(MipsCpu.opcode&0x3E00000)>>21]();}
void    pCOP1()		{	pcop1_opc[((MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->opcode)&0x3E00000)>>21]();}

void    COP2()		{	MipsCpu.halt = 1;}
void    pCOP2()		{	MipsCpu.halt = 1;}

void    SPECIAL2()	{	special2_opc[(MipsCpu.opcode  & 0x3F)](); }
void    pSPECIAL2()	{	pspecial2_opc[((MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->opcode)  & 0x3F)](); }



/*
Add Word
To add 32-bit integers. If an overflow occurs, then trap.
*/
void   ADD     () {
	_u64 temp;

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: add	%s,%s,%s", MipsCpu.PC, debug_r5900_reg[CURRENT_RD],debug_r5900_reg[CURRENT_RT],debug_r5900_reg[CURRENT_RS] );

	if ( IsSignExtended( MipsCpu.GPR[CURRENT_RS].lo ) & IsSignExtended( MipsCpu.GPR[CURRENT_RT].lo ) ) {
		
		temp = MipsCpu.GPR[CURRENT_RS].lo + MipsCpu.GPR[CURRENT_RT].lo;
		
		if ( IsNot32bitIntegerOverflow ( temp ) ) {
		   sign_extended(temp);
		   MipsCpu.GPR[CURRENT_RD].lo = temp;
		}
		else {
			//SignalExeption(IntegerOverflow)
		}
	}
	
	INCREMENT_PC();
}


void   pADD     () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = ADD;
	PrefetchExtractrt(); 
	PrefetchExtractrs();
	PrefetchExtractrd();
}



/*
Add Immediate Word
To add 32-bit integers. If an overflow occurs, then trap.
*/
void    ADDI   () {
	_u64 temp;

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: addi %s,%s,%4llx", MipsCpu.PC, debug_r5900_reg[CURRENT_RT],debug_r5900_reg[CURRENT_RS],CURRENT_IMM );

    if ( IsSignExtended( MipsCpu.GPR[CURRENT_RS].lo ) ) {
	
		sign_extendedh(CURRENT_IMM);
		temp = MipsCpu.GPR[CURRENT_RS].lo + CURRENT_IMM;
		
		if ( (_u32)temp > 0xFFFFFFFF ) {
				
				//SignalExeption(IntegerOverflow)
			
		} else {	   
			
				sign_extended(temp);
				MipsCpu.GPR[CURRENT_RT].lo = temp;
		}
	}
	INCREMENT_PC();
}

void    pADDI   () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = ADDI;
	
	PrefetchExtractrt(); 
	PrefetchExtractrs();
	PrefetchExtractimm();

}



/*
Add Immediate Unsigned Word
To add a constant to a 32-bit integer
*/
void    ADDIU () {
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: addiu %s,%s,%4llx", MipsCpu.PC, debug_r5900_reg[CURRENT_RT],debug_r5900_reg[CURRENT_RS],CURRENT_IMM );

    if ( IsSignExtended( MipsCpu.GPR[CURRENT_RS].lo ) ) {
	
		//sign_extendedh(imm);
	
	 	MipsCpu.GPR[CURRENT_RT].lo = MipsCpu.GPR[CURRENT_RS].lo + CURRENT_IMM;
		sign_extended(MipsCpu.GPR[CURRENT_RT].lo);
	    
    }
    
    INCREMENT_PC();
}


void    pADDIU () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = ADDIU;
	PrefetchExtractrt(); 
	PrefetchExtractrs();
	PrefetchExtractimm();

	sign_extendedh(MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->imm);

}


/*
Add Unsigned Word
To add 32-bit integers
*/
void   ADDU    () {

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: addu %s,%s,%s", MipsCpu.PC, debug_r5900_reg[CURRENT_RD],debug_r5900_reg[CURRENT_RT],debug_r5900_reg[CURRENT_RS] );
	
    if ( IsSignExtended( MipsCpu.GPR[CURRENT_RS].lo ) & IsSignExtended( MipsCpu.GPR[CURRENT_RT].lo ) ) {
		
		MipsCpu.GPR[CURRENT_RD].lo = MipsCpu.GPR[CURRENT_RS].lo + MipsCpu.GPR[CURRENT_RT].lo;
		sign_extended(MipsCpu.GPR[CURRENT_RD].lo);
	
	}
	
	INCREMENT_PC();
}

void   pADDU    () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = ADDU;
	
	PrefetchExtractrt(); 
	PrefetchExtractrs();
	PrefetchExtractrd();

}



/*
And
To do a bitwise logical AND
*/
void   AND () {
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: and	%s,%s,%s", MipsCpu.PC, debug_r5900_reg[CURRENT_RD],debug_r5900_reg[CURRENT_RS],debug_r5900_reg[CURRENT_RT] );
	
	MipsCpu.GPR[CURRENT_RD].lo = MipsCpu.GPR[CURRENT_RS].lo & MipsCpu.GPR[CURRENT_RT].lo;
	
	INCREMENT_PC();
}

void   pAND () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = AND;
	PrefetchExtractrt(); 
	PrefetchExtractrs();
	PrefetchExtractrd();
	

}



/*
And Immediate
To do a bitwise logical AND with a constant
*/
void    ANDI() {
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: andi %s,%s,%4llx", MipsCpu.PC, debug_r5900_reg[CURRENT_RT],debug_r5900_reg[CURRENT_RS],CURRENT_IMM );
	
	MipsCpu.GPR[CURRENT_RT].lo = MipsCpu.GPR[CURRENT_RS].lo & CURRENT_IMM;
	
	INCREMENT_PC();
}

void    pANDI() {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = ANDI;
	PrefetchExtractrt(); 
	PrefetchExtractrs();
	PrefetchExtractimm();
	

}




/*
Branch on Equal
To compare GPRs then do a PC-relative conditional branch
*/
void     BEQ () {
	_u64	target_branch;

	//offset = offset << 2;
	//sign_extendedh(offset);
	target_branch = MipsCpu.PC /*+ 4*/ +  CURRENT_OFFSET ;
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: beq %s,%s,%4llx", MipsCpu.PC, debug_r5900_reg[CURRENT_RS],debug_r5900_reg[CURRENT_RT],target_branch );
	
	if ( MipsCpu.GPR[CURRENT_RS].lo == MipsCpu.GPR[CURRENT_RT].lo ) {
		if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );
		
        INCREMENT_PC();
		MipsUpdateCpu();
		
        MipsCpu.PC = target_branch;
	}
	else INCREMENT_PC();
} 

void     pBEQ () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = BEQ;	
	
	PrefetchExtractrs();
	PrefetchExtractrt();
	PrefetchExtractoffset();
	
	MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset = MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset << 2;
	sign_extendedh(MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset);
	MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset = MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset + 4;

} 




/*
Branch on Equal Likely
To compare GPRs then do a PC-relative conditional branch; execute the delay slot only if the branch is taken.
*/
void     BEQL() {
	_u64	target_branch;

	//offset = offset << 2;
	//sign_extendedh(offset);
	target_branch = MipsCpu.PC /*+ 4*/ +  CURRENT_OFFSET ;
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: beql %s,%s,%4llx", MipsCpu.PC, debug_r5900_reg[CURRENT_RS],debug_r5900_reg[CURRENT_RT],target_branch );
	
	if ( MipsCpu.GPR[CURRENT_RS].lo == MipsCpu.GPR[CURRENT_RT].lo ) {
		if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );
		INCREMENT_PC();
		MipsUpdateCpu();
		MipsCpu.PC = target_branch;
	}
	else MipsCpu.PC = MipsCpu.PC + 8;
}


void     pBEQL() {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = BEQL;
	
	PrefetchExtractrs();
	PrefetchExtractrt();
	PrefetchExtractoffset();
	
	MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset = MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset << 2;
	sign_extendedh(MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset);
	MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset = MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset + 4;
	

}


/*
Branch on Greater Than or Equal to Zero
To test a GPR then do a PC-relative conditional branch
*/
void      BGEZ() {
	_u64	target_branch;


	//offset = offset << 2;
	//sign_extendedh(offset);
	target_branch = MipsCpu.PC /* + 4*/ +  CURRENT_OFFSET ;
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: bgez %s,%4llx", MipsCpu.PC, debug_r5900_reg[CURRENT_RS], target_branch );
	

	if ( (_s64)(MipsCpu.GPR[CURRENT_RS].lo) >= 0 ) {
		if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );
		INCREMENT_PC();
		MipsUpdateCpu();
		MipsCpu.PC = target_branch;
	}
	else INCREMENT_PC();
}

void      pBGEZ() {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = BGEZ;	
	
	PrefetchExtractrs();
	PrefetchExtractoffset();

	MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset = MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset << 2;
	sign_extendedh(MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset);
	MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset = MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset + 4;


}



void      BGEZAL () {	MipsCpu.halt = 1;}
void      pBGEZAL () {	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = BGEZAL;	}

void      BGEZALL() {	MipsCpu.halt = 1;}
void      pBGEZALL() {		MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = BGEZALL;}

/*
Branch on Greater Than or Equal to Zero Likely
To test a GPR then do a PC-relative conditional branch; execute the delay slot only if the branch is taken.
*/
void      BGEZL  () {		
	_u64	target_branch;
	
	//offset = offset << 2;
	//sign_extendedh(offset);
	target_branch = MipsCpu.PC /*+ 4*/ +  CURRENT_OFFSET ;
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: bgezl %s,%4llx", MipsCpu.PC,debug_r5900_reg[CURRENT_RS],target_branch );

	if ( (_s64)MipsCpu.GPR[CURRENT_RS].lo >= 0 ) {
		if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );
		INCREMENT_PC();
		MipsUpdateCpu();
		MipsCpu.PC = target_branch;
	}
	else MipsCpu.PC = MipsCpu.PC + 8;
}


void      pBGEZL  () {		
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = BGEZL;
	PrefetchExtractrs();
	PrefetchExtractoffset();
	
	MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset = MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset << 2;
	sign_extendedh(MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset);
	MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset = MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset + 4;
	
	

}



/*
Branch on Greater Than Zero
To test a GPR then do a PC-relative conditional branch
*/
void     BGTZ () {
	_u64	target_branch;

	//offset = offset << 2;
	//sign_extendedh(offset);
	target_branch = MipsCpu.PC /*+ 4*/ +  CURRENT_OFFSET ;
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: bgtz %s,%4llx", MipsCpu.PC, debug_r5900_reg[CURRENT_RS], target_branch );


	if ( (_s64)( MipsCpu.GPR[CURRENT_RS].lo ) > 0 ) {
		if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );
		INCREMENT_PC();
		MipsUpdateCpu();
		MipsCpu.PC = target_branch;
	}
	else INCREMENT_PC(); 
}


void     pBGTZ () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = BGTZ;
	
	PrefetchExtractrs();
	PrefetchExtractoffset();
	
	MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset = MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset << 2;
	sign_extendedh(MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset);
	MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset = MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset + 4;

	

}


void    BGTZL() {	MipsCpu.halt = 1;}
void    pBGTZL() {		MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = BGTZL;}


/*
Branch on Less Than or Equal to Zero
To test a GPR then do a PC-relative conditional branch
*/
void     BLEZ () {
	_u64	target_branch;
	
	//offset = offset << 2;
	//sign_extendedh(offset);
	target_branch = MipsCpu.PC /*+ 4*/ +  CURRENT_OFFSET ;
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: blez %s,%4llx", MipsCpu.PC, debug_r5900_reg[CURRENT_RS], target_branch );
	
	if ( (_s64)(MipsCpu.GPR[CURRENT_RS].lo) <= 0 ) {
		if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );
		INCREMENT_PC();
		MipsUpdateCpu();
		MipsCpu.PC = target_branch;
	}
	else INCREMENT_PC();
}

void     pBLEZ () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = BLEZ;
	PrefetchExtractrs();
	PrefetchExtractoffset();
	
	MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset = MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset << 2;
	sign_extendedh(MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset);
	MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset = MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset + 4;
	

}



/*
Branch on Less Than or Equal to Zero Likely
To test a GPR then do a PC-relative conditional branch; execute the delay slot only if the branch is taken.
*/
void    BLEZL() {		
	_u64	target_branch;
	
	//offset = offset << 2;
	//sign_extendedh(offset);
	target_branch = MipsCpu.PC /*+ 4*/ +  CURRENT_OFFSET ;
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: blezl %s,%4llx", MipsCpu.PC,debug_r5900_reg[CURRENT_RS],target_branch );
	
	if ( (_s64)MipsCpu.GPR[CURRENT_RS].lo <= 0 ) {
		if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );
		INCREMENT_PC();
		MipsUpdateCpu();
		MipsCpu.PC = target_branch;
	}
	else MipsCpu.PC = MipsCpu.PC + 8;
}

void    pBLEZL() {		
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = BLEZL;
	
	PrefetchExtractrs();
	PrefetchExtractoffset();
	

	MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset = MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset << 2;
	sign_extendedh(MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset);
	MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset = MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset + 4;
	

}



/*
Branch on Less Than Zero Likely
To test a GPR then do a PC-relative conditional branch; execute the delay slot only if the branch is taken.
*/
void	BLTZ() {
	_u64	target_branch;

	//offset = offset << 2;
	//sign_extendedh(offset);
	target_branch = MipsCpu.PC /*+ 4 */+  CURRENT_OFFSET ;
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: bltz %s,%4llx", MipsCpu.PC, debug_r5900_reg[CURRENT_RS], target_branch );

	
	if ((_s64)( MipsCpu.GPR[CURRENT_RS].lo) < 0 ) {
		if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );
		INCREMENT_PC();
		MipsUpdateCpu();
		MipsCpu.PC = target_branch;
	}
	else INCREMENT_PC();
}


void	pBLTZ() {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = BLTZ;
	
	PrefetchExtractrs();
	PrefetchExtractoffset();
	
	MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset = MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset << 2;
	sign_extendedh(MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset);
	MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset = MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset + 4;

}


void      BLTZALL() {	MipsCpu.halt = 1;}
void      pBLTZALL() {	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = BLTZALL;	}


void      BLTZAL () {	MipsCpu.halt = 1;}
void      pBLTZAL () {		MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = BLTZAL;}


/*
Branch on Less Than Zero Likely
*/
void      BLTZL  () {		
	_u64	target_branch;
	
	//offset = offset << 2;
	//sign_extendedh(offset);
	target_branch = MipsCpu.PC /*+ 4*/ +  CURRENT_OFFSET ;
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: bltzl %s,%4llx", MipsCpu.PC,debug_r5900_reg[CURRENT_RS],target_branch );


	if ( (_s64)MipsCpu.GPR[CURRENT_RS].lo < 0 ) {
		if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );
		INCREMENT_PC();
		MipsUpdateCpu();
		MipsCpu.PC = target_branch;
	}
	else MipsCpu.PC = MipsCpu.PC + 8;
}


void      pBLTZL  () {		
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = BLTZL;
	
	PrefetchExtractrs();
	PrefetchExtractoffset();

	MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset = MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset << 2;
	sign_extendedh(MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset);
	MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset = MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset + 4;
	

}



/*
Branch on Not Equal
To compare GPRs then do a PC-relative conditional branch
*/
void     BNE () {
	_u64	target_branch;
	
	//offset = offset << 2;
	//sign_extendedh(offset);
	target_branch = MipsCpu.PC /*+ 4*/ +  CURRENT_OFFSET ;
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: bne %s,%s,%4llx", MipsCpu.PC, debug_r5900_reg[CURRENT_RS],debug_r5900_reg[CURRENT_RT],target_branch );
	
	if ( MipsCpu.GPR[CURRENT_RS].lo != MipsCpu.GPR[CURRENT_RT].lo ) {
		if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );
		INCREMENT_PC();
		MipsUpdateCpu();
		MipsCpu.PC = target_branch;
	}
	else INCREMENT_PC();
}

void     pBNE () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = BNE;	
	PrefetchExtractrs();
	PrefetchExtractrt();
	PrefetchExtractoffset();

	MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset = MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset << 2;
	sign_extendedh(MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset);
	MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset = MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset + 4;

	

}



/*
Branch on Not Equal Likely
To compare GPRs then do a PC-relative conditional branch; execute the delay slot only if the branch is taken.
*/
void     BNEL() {
	_u64	target_branch;
	
	//offset = offset << 2;
	//sign_extendedh(offset);
	target_branch = MipsCpu.PC /*+ 4*/ +  CURRENT_OFFSET ;
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: bnel %s,%s,%4llx", MipsCpu.PC, debug_r5900_reg[CURRENT_RS],debug_r5900_reg[CURRENT_RT],target_branch );
	
	if ( MipsCpu.GPR[CURRENT_RS].lo != MipsCpu.GPR[CURRENT_RT].lo ) {
		if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );
		INCREMENT_PC();
		MipsUpdateCpu();
		MipsCpu.PC = target_branch;
	}
	else MipsCpu.PC = MipsCpu.PC + 8;
}

void     pBNEL() {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = BNEL;	
	
	PrefetchExtractrs();
	PrefetchExtractrt();
	PrefetchExtractoffset();

	MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset = MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset << 2;
	sign_extendedh(MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset);
	MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset = MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset + 4;

	

}

 
/*
Breakpoint
To cause a Breakpoint exception
*/
void	BREAK()	{	MipsCpu.halt = 1;}
void	pBREAK()	{	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = BREAK;}

void	DADD()	{	MipsCpu.halt = 1;}
void	pDADD()	{	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = DADD;	}

void	DADDI()	{	MipsCpu.halt = 1;}
void	pDADDI()	{	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = DADDI;	}



/*
Doubleword Add Immediate Unsigned
To add a constant to a 64-bit integer
*/
void    DADDIU    () {
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: daddiu %s,%s,%4llx", MipsCpu.PC, debug_r5900_reg[CURRENT_RT],debug_r5900_reg[CURRENT_RS],CURRENT_IMM );
	
	//sign_extendedh(imm);
	MipsCpu.GPR[CURRENT_RT].lo = MipsCpu.GPR[CURRENT_RS].lo + CURRENT_IMM;

	INCREMENT_PC();
}

void    pDADDIU    () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = DADDIU;  

	PrefetchExtractrt(); 
	PrefetchExtractrs();
	PrefetchExtractimm();
	
	sign_extendedh(MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->imm);

}



/*
Doubleword Add Unsigned
To add 64-bit integers
*/
void   DADDU  () {
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: daddu %s,%s,%s", MipsCpu.PC, debug_r5900_reg[CURRENT_RD], debug_r5900_reg[CURRENT_RS], debug_r5900_reg[CURRENT_RT]);
	
	MipsCpu.GPR[CURRENT_RD].lo = MipsCpu.GPR[CURRENT_RS].lo + MipsCpu.GPR[CURRENT_RT].lo;
	
	INCREMENT_PC();
}

void   pDADDU  () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = DADDU;
	PrefetchExtractrs();
	PrefetchExtractrt();
	PrefetchExtractrd();
	

}



/*
Divide Word
To divide a 32-bit signed integers
*/
void   DIV  () {		
	ldiv_t result_div;
	
	if ( debug_op == 1 )  	sprintf(debugTempText,"%8.8llx: div %s,%s", MipsCpu.PC, debug_r5900_reg[CURRENT_RS],debug_r5900_reg[CURRENT_RT]);

    if ( IsSignExtended( MipsCpu.GPR[CURRENT_RS].lo ) & IsSignExtended( MipsCpu.GPR[CURRENT_RT].lo ) & ( MipsCpu.GPR[CURRENT_RT].lo != 0 )  ) {   
		
		result_div = ldiv( (_s32) (MipsCpu.GPR[CURRENT_RS].lo & 0xFFFFFFFF )  , (_s32) (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF )  );
		MipsCpu.GPR[CPU_REG_LO].lo = result_div.quot;
		MipsCpu.GPR[CPU_REG_HI].lo = result_div.rem;
		
		sign_extended( MipsCpu.GPR[CPU_REG_LO].lo);
		sign_extended( MipsCpu.GPR[CPU_REG_HI].lo);
		
	}
	
	INCREMENT_PC();
}


void   pDIV  () {		
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = DIV;
	PrefetchExtractrs();
	PrefetchExtractrt();
	

}



/*
Divide Unsigned Word
To divide a 32-bit unsigned integers
*/
void   DIVU () {
	ldiv_t result_div;

	if ( debug_op == 1 )  	sprintf(debugTempText,"%8.8llx: divu %s,%s", MipsCpu.PC, debug_r5900_reg[CURRENT_RS],debug_r5900_reg[CURRENT_RT]);

    if ( IsSignExtended( MipsCpu.GPR[CURRENT_RS].lo ) & IsSignExtended( MipsCpu.GPR[CURRENT_RT].lo ) & ( MipsCpu.GPR[CURRENT_RT].lo != 0 )  ) {   
		
		result_div = ldiv( (_u32)  (MipsCpu.GPR[CURRENT_RS].lo & 0xFFFFFFFF )  , (_u32) (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF )  );
		MipsCpu.GPR[CPU_REG_LO].lo = result_div.quot;
		MipsCpu.GPR[CPU_REG_HI].lo = result_div.rem;
		
		sign_extended( MipsCpu.GPR[CPU_REG_LO].lo);
		sign_extended( MipsCpu.GPR[CPU_REG_HI].lo);
    
    }
	
	INCREMENT_PC();
}

void   pDIVU () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = DIVU;
	PrefetchExtractrs();
	PrefetchExtractrt();
	

}


/*
Doubleword Shift Left Logical Variable
To execute a left-shift of a doubleword by a variable number of bits
*/
void   DSLL     () {
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: dsll %s,%s,%4lx", MipsCpu.PC, debug_r5900_reg[CURRENT_RD], debug_r5900_reg[CURRENT_RT],CURRENT_SA );
	
	MipsCpu.GPR[CURRENT_RD].lo = MipsCpu.GPR[CURRENT_RT].lo << CURRENT_SA ;
	
	INCREMENT_PC();
}

void   pDSLL     () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = DSLL;
	PrefetchExtractrt();
	PrefetchExtractrd();
	PrefetchExtractsa();
	

}


/*
Doubleword Shift Left Logical Plus 32
To execute a left-shift of a doubleword by a fixed amount 32 to 63 bits
*/
void   DSLL32  () {

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: dsll32 %s,%s,%4lx", MipsCpu.PC, debug_r5900_reg[CURRENT_RD], debug_r5900_reg[CURRENT_RT],CURRENT_SA );
	
	MipsCpu.GPR[CURRENT_RD].lo = MipsCpu.GPR[CURRENT_RT].lo <<  CURRENT_SA  ;
	
	INCREMENT_PC();
}

void   pDSLL32  () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = DSLL32;
	PrefetchExtractrt();
	PrefetchExtractrd();
	PrefetchExtractsa();
	
	MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->sa = MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->sa + 32;

}


void   DSLLV () {	MipsCpu.halt = 1;}
void   pDSLLV () {	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = DSLLV;	}


void   DSRA   () {	MipsCpu.halt = 1;}
void   pDSRA   () {		MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = DSRA;}

/*
Doubleword Shift Right Arithmetic Plus 32
To execute an arithmetic right-shift of a doubleword by a fixed amount 32 to 63 bits
*/
void   DSRA32() {
	_u8	 boucle;	
	_u64 temp;

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: dsra32 %s,%s,%4lx", MipsCpu.PC, debug_r5900_reg[CURRENT_RD], debug_r5900_reg[CURRENT_RT],CURRENT_SA );
	
	if ((MipsCpu.GPR[CURRENT_RT].lo & SIGN_BIT_MASK_64) == SIGN_BIT_MASK_64 ) { 
		temp = MipsCpu.GPR[CURRENT_RT].lo;
		for ( boucle = 0 ; boucle < CURRENT_SA  ; boucle++ ) temp = ( temp >> 1 ) | SIGN_BIT_MASK_64 ;
	
	} else 	temp = MipsCpu.GPR[CURRENT_RT].lo >>  CURRENT_SA  ;

	MipsCpu.GPR[CURRENT_RD].lo = temp;
	INCREMENT_PC();
}

void   pDSRA32() {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = DSRA32;
	PrefetchExtractrt();
	PrefetchExtractrd();
	PrefetchExtractsa();

	MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->sa = MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->sa + 32;
}



void   DSRAV () {	MipsCpu.halt = 1;}
void   pDSRAV () {		MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = DSRAV;
}

/*
Doubleword Shift Right Logical DSRL
To execute a logical right-shift of a doubleword by a fixed amount 0 to 31 bits
*/
void   DSRL  () {
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: dsrl %s,%s,%4lx", MipsCpu.PC, debug_r5900_reg[CURRENT_RD], debug_r5900_reg[CURRENT_RT],CURRENT_SA );
	
	MipsCpu.GPR[CURRENT_RD].lo = MipsCpu.GPR[CURRENT_RT].lo >> CURRENT_SA ;
	
	INCREMENT_PC();
}

void   pDSRL  () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = DSRL;
	PrefetchExtractrt();
	PrefetchExtractrd();
	PrefetchExtractsa();
	

}


/*
Doubleword Shift Right Logical Plus 32
To execute a logical right-shift of a doubleword by a fixed amount 32 to 63 bits
*/
void   DSRL32   () {
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: dsrl32 %s,%s,%4lx", MipsCpu.PC, debug_r5900_reg[CURRENT_RD], debug_r5900_reg[CURRENT_RT],CURRENT_SA );
	
	MipsCpu.GPR[CURRENT_RD].lo = MipsCpu.GPR[CURRENT_RT].lo >>  CURRENT_SA  ;
	
	INCREMENT_PC();
}

void   pDSRL32   () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = DSRL32;
	PrefetchExtractrt();
	PrefetchExtractrd();
	PrefetchExtractsa();

	MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->sa = MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->sa + 32;
	
}

/*

*/

void   DSRLV () {	
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: srlv %s,%s,%s", MipsCpu.PC, debug_r5900_reg[CURRENT_RD], debug_r5900_reg[CURRENT_RT], debug_r5900_reg[CURRENT_RS] );
	

		
	MipsCpu.GPR[CURRENT_RD].lo =  MipsCpu.GPR[CURRENT_RT].lo   >>  ( MipsCpu.GPR[CURRENT_RS].lo & 0x3F);	
		
	

	
	INCREMENT_PC();



}

void   pDSRLV () {	
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = DSRLV;

	PrefetchExtractrt();
	PrefetchExtractrd();
	PrefetchExtractrs();
	
}

/*
*/

void   DSUB  () {	MipsCpu.halt = 1;}
void   pDSUB  () {		MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = DSUB;
}

/*
Doubleword Subtract Unsigned
To subtract 64-bit integers
*/
void   DSUBU() {

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: dsubu %s,%s,%s", MipsCpu.PC, debug_r5900_reg[CURRENT_RD],debug_r5900_reg[CURRENT_RS],debug_r5900_reg[CURRENT_RT] );
	
	MipsCpu.GPR[CURRENT_RD].lo = MipsCpu.GPR[CURRENT_RS].lo - MipsCpu.GPR[CURRENT_RT].lo;
	
	INCREMENT_PC();
}


void   pDSUBU() {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = DSUBU;
	PrefetchExtractrt(); 
	PrefetchExtractrs();
	PrefetchExtractrd();

}


/*
Jump
To branch within the current 256 MB-aligned region
*/
void J() {
	_u64 	target_branch;
	
	target_branch = CURRENT_INSTR_INDEX ;
	
	if ( debug_op == 1 ) 	{
		sprintf(debugTempText,"%8.8llx: j %8llx", MipsCpu.PC, target_branch );
		PrintToOpcodeOutput ( debugTempText );
	}
	
	INCREMENT_PC();
	MipsUpdateCpu();
	
	MipsCpu.PC =  target_branch;
}

void pJ() {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = J;

	PrefetchExtractinstr_index();
	
	MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->instr_index = MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->instr_index << 2;

}


/*
Jump and Link
To execute a procedure call within the current 256 MB-aligned region
*/
void     JAL  () {
	_u64 	target_branch;
	
	MipsCpu.GPR[ CPU_REG_RA ].lo =  MipsCpu.PC + 8;
	target_branch = CURRENT_INSTR_INDEX /* << 2*/;
	
	if ( debug_op == 1 ) 	{
		sprintf(debugTempText,"%8.8llx: jal %8llx", MipsCpu.PC, target_branch );
		PrintToOpcodeOutput ( debugTempText );
	}
	
	INCREMENT_PC();
	MipsUpdateCpu();
	
	MipsCpu.PC =  target_branch;
}

void     pJAL  () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = JAL;

	PrefetchExtractinstr_index();
	
	MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->instr_index = MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->instr_index << 2;

}


/*
Jump and Link Register
To execute a procedure call to an instruction address in a register
*/
void   JALR () {
	_u64	branch_target;
	
	branch_target = MipsCpu.GPR[CURRENT_RS].lo;
	MipsCpu.GPR[CURRENT_RD].lo = MipsCpu.PC + 8;
	
	if ( debug_op == 1 ) {
	    sprintf(debugTempText,"%8.8llx: jalr %s,%s with %s=%8llx", MipsCpu.PC, debug_r5900_reg[CURRENT_RD],debug_r5900_reg[CURRENT_RS] ,debug_r5900_reg[CURRENT_RS], MipsCpu.GPR[CURRENT_RS].lo );
	    PrintToOpcodeOutput ( debugTempText );
	}
	
	INCREMENT_PC();
	MipsUpdateCpu();
	
	MipsCpu.PC =  branch_target;
}

void   pJALR () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = JALR;
	
	PrefetchExtractrs();
	PrefetchExtractrd();
	

}


/*
Jump Register
To execute a branch to an instruction address in a register
*/
void   JR   () {
	_u64	branch_target;
	
	if ( debug_op == 1 ) {
	     sprintf(debugTempText,"%8.8llx: jr %s with %s=%8llx", MipsCpu.PC, debug_r5900_reg[CURRENT_RS],debug_r5900_reg[CURRENT_RS] , MipsCpu.GPR[CURRENT_RS].lo );
	     PrintToOpcodeOutput ( debugTempText );
	}
	
	branch_target = MipsCpu.GPR[CURRENT_RS].lo;
	
	INCREMENT_PC();
	MipsUpdateCpu();
	
	MipsCpu.PC =  branch_target;
}

void   pJR   () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = JR;
	
	PrefetchExtractrs();
	

}


/*
Load Byte
To load a byte from memory as a signed value
*/
void    LB     () {
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: lb %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[CURRENT_RT],CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );
	
	//sign_extendedh(offset);
	MipsMemoryRead ( &MipsCpu.GPR[CURRENT_RT] , DATA_B ,  MipsCpu.GPR[CURRENT_BASE].lo , CURRENT_OFFSET ) ;
	sign_extendedb( MipsCpu.GPR[CURRENT_RT].lo );
	
	INCREMENT_PC();
}

void    pLB     () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = LB;
	PrefetchExtractbase();
	PrefetchExtractrt();
	PrefetchExtractoffset();
	
	sign_extendedh(MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset);

}


/*
Load Byte Unsigned
To load a byte from memory as an unsigned value
*/
void    LBU () {

	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: lbu %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[CURRENT_RT],CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );

	//sign_extendedh(offset);
	MipsMemoryRead ( &MipsCpu.GPR[CURRENT_RT] , DATA_B ,  MipsCpu.GPR[CURRENT_BASE].lo , CURRENT_OFFSET ) ;

    INCREMENT_PC();
}

void    pLBU () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = LBU;
	PrefetchExtractbase();
	PrefetchExtractrt();
	PrefetchExtractoffset();
	
	sign_extendedh(MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset);

}

/*
Load Doubleword
To load a doubleword from memory
*/
void    LD   () {
	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: ld %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[CURRENT_RT],CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );
	
	//sign_extendedh(offset);
	MipsMemoryRead ( &MipsCpu.GPR[CURRENT_RT] , DATA_DW ,  MipsCpu.GPR[CURRENT_BASE].lo , CURRENT_OFFSET ) ;
	
	INCREMENT_PC();
}

void    pLD   () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = LD;
	PrefetchExtractbase();
	PrefetchExtractrt();
	PrefetchExtractoffset();
	
	sign_extendedh(MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset);

}


/*
Load Doubleword Left
To load the most-significant part of a doubleword from an unaligned memory address
*/
void    LDL  () {
    _u128   temp128;	
    _u64    ReadAdress;
	
	if ( debug_op == 1 )		sprintf(debugTempText,"%8.8llx: ldl %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[CURRENT_RT],CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );
	
	//sign_extendedh(offset);
 
	switch ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET ) & 0xF)
	{
		case 0:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x00FFFFFFFFFFFFFF) | (temp128.lo << 56);
		break;

		case 1:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x0000FFFFFFFFFFFF) | (temp128.lo << 48);
		break;

		case 2:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x000000FFFFFFFFFF ) | (temp128.lo << 40);
		break;
	
		case 3:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x00000000FFFFFFFF ) | (temp128.lo << 32);
		break;

		case 4:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x0000000000FFFFFF) | (temp128.lo << 24);
		break;

		case 5:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x000000000000FFFF ) | (temp128.lo << 16);
		break;
	
		case 6:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x00000000000000FF ) | (temp128.lo << 8);
		break;
	
		case 7:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = temp128.lo ;
		break;
		
		
		case 8:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x00FFFFFFFFFFFFFF) | (temp128.lo << 56);
		break;
	
		case 9:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x0000FFFFFFFFFFFF) | (temp128.lo << 48);
		break;
	
		case 10:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x000000FFFFFFFFFF ) | (temp128.lo << 40);
		break;
	
		case 11:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x00000000FFFFFFFF ) | (temp128.lo << 32);
		break;

		case 12:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x0000000000FFFFFF) | (temp128.lo << 24);
		break;

		case 13:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x000000000000FFFF ) | (temp128.lo << 16);
		break;

		case 14:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x00000000000000FF ) | (temp128.lo << 8);
		break;

		case 15:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = temp128.lo ;
		break;
     }

	INCREMENT_PC();
}

void    pLDL  () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = LDL;
 
    PrefetchExtractbase();
	PrefetchExtractrt();
	PrefetchExtractoffset();

	sign_extendedh(MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset);
	

}



/*
Load Doubleword Right
To load the least-significant part of a doubleword from an unaligned memory address
*/
void    LDR   () {
	_u128  temp128;
    _u64    ReadAdress;

	if ( debug_op == 1 )		sprintf(debugTempText,"%8.8llx: ldr %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[CURRENT_RT],CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );

	//sign_extendedh(offset);

	switch ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET ) & 0xF)
	{
		case 0:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = temp128.lo;
		break;
		
		case 1:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFF00000000000000) | (temp128.lo >> 8);
		break;
		
		case 2:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFF000000000000) | (temp128.lo >> 16);
		break;
		
		case 3:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFF0000000000) | (temp128.lo >> 24);
		break;
		
		case 4:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF00000000) | (temp128.lo >> 32);
		break;
		
		case 5:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFFFF000000) | (temp128.lo >> 40);
		break;
		
		case 6:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFFFFFF0000) | (temp128.lo >> 48);
		break;
		
		case 7:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFFFFFFFF00) | (temp128.lo >> 56);
		break;
		
		case 8:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = temp128.lo;
		break;
		
		case 9:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFF00000000000000) | (temp128.lo >> 8);
		break;
		
		case 10:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFF000000000000) | (temp128.lo >> 16);
		break;
		
		case 11:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFF0000000000) | (temp128.lo >> 24);
		break;
		
		case 12:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF00000000) | (temp128.lo >> 32);
		break;
		
		case 13:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFFFF000000) | (temp128.lo >> 40);
		break;
		
		case 14:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFFFFFF0000) | (temp128.lo >> 48);
		break;
		
		case 15:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFFFFFFFF00) | (temp128.lo >> 56);
		break;	
	}


	INCREMENT_PC();
}

void    pLDR   () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = LDR;

     
    PrefetchExtractbase();
	PrefetchExtractrt();
	PrefetchExtractoffset();

	sign_extendedh(MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset);
	

}




/*
Load Halfword
To load a halfword from memory as a signed value
*/
void    LH    () {

	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: lh %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[CURRENT_RT],CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );
	
	//sign_extendedh(offset);
	MipsMemoryRead ( &MipsCpu.GPR[CURRENT_RT] , DATA_HW ,  MipsCpu.GPR[CURRENT_BASE].lo , CURRENT_OFFSET ) ;
	sign_extendedh( MipsCpu.GPR[CURRENT_RT].lo );
	
	INCREMENT_PC();
}

void    pLH    () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = LH;
	PrefetchExtractbase();
	PrefetchExtractrt();
	PrefetchExtractoffset();

	sign_extendedh(MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset);
	

}

/*
Load Halfword Unsigned
To load a halfword from memory as an unsigned value
*/
void    LHU () {
	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: lhu %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[CURRENT_RT],CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );
	
	//sign_extendedh(offset);
	MipsMemoryRead ( &MipsCpu.GPR[CURRENT_RT] , DATA_HW ,  MipsCpu.GPR[CURRENT_BASE].lo , CURRENT_OFFSET ) ;
	
	INCREMENT_PC();
}

void    pLHU () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = LHU;
	PrefetchExtractbase();
	PrefetchExtractrt();
	PrefetchExtractoffset();
	
	sign_extendedh(MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset);

}


/*
Load Upper Immediate
To load a constant into the upper half of a word
*/
void    LUI  () {

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: lui %s,%4llx", MipsCpu.PC, debug_r5900_reg[CURRENT_RT],CURRENT_IMM );
  
    MipsCpu.GPR[CURRENT_RT].lo = CURRENT_IMM /*<< 16*/;
 	sign_extended( MipsCpu.GPR[CURRENT_RT].lo );
 	
    INCREMENT_PC();
}

void    pLUI  () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = LUI;
	PrefetchExtractrt(); 
	PrefetchExtractimm();

	MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->imm = MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->imm << 16;
}


/*
Load Word
To load a word from memory as a signed value
*/
void    LW   () {
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: lw %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[CURRENT_RT],CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );

	//sign_extendedh(offset);
	MipsMemoryRead ( &MipsCpu.GPR[CURRENT_RT] , DATA_W ,  MipsCpu.GPR[CURRENT_BASE].lo , CURRENT_OFFSET ) ;
	sign_extended( MipsCpu.GPR[CURRENT_RT].lo );

    INCREMENT_PC();
}

void    pLW   () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = LW;
	PrefetchExtractbase();
	PrefetchExtractrt();
	PrefetchExtractoffset();
	
	sign_extendedh(MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset);


}


/*
Load Word Left
To load the most-significant part of a word as a signed value from an unaligned memory address
*/
void    LWL2 () { 	

	_u128	temp128;
	_u64	temp64;
	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: lwl %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[CURRENT_RT],CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );
 
    printf("LWL\n");

	sign_extendedh(CURRENT_OFFSET);
	MipsMemoryRead ( &temp128 , DATA_W ,  MipsCpu.GPR[CURRENT_BASE].lo , CURRENT_OFFSET ) ;

	temp64 =  temp128.lo   & 0xFFFF0000;
	MipsCpu.GPR[CURRENT_RT].lo = ( MipsCpu.GPR[CURRENT_RT].lo & 0x0000FFFF )  | temp64;
	
	sign_extended( MipsCpu.GPR[CURRENT_RT].lo );
  
    INCREMENT_PC();
    

}

void    LWL  () {
    _u128   temp128;	
    _u64    ReadAdress;
	
	if ( debug_op == 1 )		sprintf(debugTempText,"%8.8llx: ldl %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[CURRENT_RT],CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );
	
	//sign_extendedh(offset);
 
	switch ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET ) & 0xF)
	{
		case 0:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x00FFFFFF) | ((temp128.lo << 24) & 0xFF000000);
		break;

		case 1:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x0000FFFF) | ((temp128.lo << 16) & 0xFFFF0000);
		break;

		case 2:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x000000FF ) | ((temp128.lo << 8) & 0xFFFFFF00);
		break;
	
		case 3:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = temp128.lo & 0xFFFFFFFF;
		break;

		case 4:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x00FFFFFF) | ((temp128.lo >> 8 ) & 0xFF00000000);
		break;

		case 5:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x0000FFFF) | ((temp128.lo >> 16) & 0xFFFF0000);
		break;

		case 6:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x000000FF ) | ((temp128.lo >> 24) & 0xFFFFFF00);
		break;
	
		case 7:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (temp128.lo >> 32) & 0xFFFFFFFF;
		break;

		case 8:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 8 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x00FFFFFF) | ((temp128.lo << 24) & 0xFF000000);
		break;

		case 9:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 8 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x0000FFFF) | ((temp128.lo << 16) & 0xFFFF0000);
		break;

		case 10:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 8 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x000000FF ) | ((temp128.lo << 8) & 0xFFFFFF00);
		break;
	
		case 11:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 8 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = temp128.lo & 0xFFFFFFFF;
		break;

		case 12:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 8 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x00FFFFFF) | ((temp128.lo >> 8 ) & 0xFF00000000);
		break;

		case 13:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 8 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x0000FFFF) | ((temp128.lo >> 16) & 0xFFFF0000);
		break;

		case 14:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 8 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x000000FF ) | ((temp128.lo >> 24) & 0xFFFFFF00);
		break;
	
		case 15:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 8 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (temp128.lo >> 32) & 0xFFFFFFFF;
		break;




     }

	INCREMENT_PC();
}


void    pLWL () { 	
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = LWL;



	PrefetchExtractbase();
	PrefetchExtractrt();
	PrefetchExtractoffset();
	
 	sign_extendedh(MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset);

}


/*
Load Word Right
To load the least-significant part of a word from an unaligned memory address as a signed value
*/
void    LWR2  () {		

	_u128	temp128;
	_u64	temp64;
	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: lwr %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[CURRENT_RT],CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );

    printf("LWR\n");

	sign_extendedh(CURRENT_OFFSET);
	MipsMemoryRead ( &temp128 , DATA_W ,  MipsCpu.GPR[CURRENT_BASE].lo , CURRENT_OFFSET ) ;
	
		
	temp64 = ( temp128.lo  ) & 0xFFFF;
	
	MipsCpu.GPR[CURRENT_RT].lo = ( MipsCpu.GPR[CURRENT_RT].lo & 0xFFFF0000 )  | temp64;
	
	sign_extended( MipsCpu.GPR[CURRENT_RT].lo );
	
	

    INCREMENT_PC();
	
}


void    LWR   () {
	_u128  temp128;
    _u64    ReadAdress;

	if ( debug_op == 1 )		sprintf(debugTempText,"%8.8llx: ldr %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[CURRENT_RT],CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );

	//sign_extendedh(offset);

	switch ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET ) & 0xF)
	{
		case 0:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = temp128.lo & 0xFFFFFFFF;
		break;
		
		case 1:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFFFF000000) | ((temp128.lo >> 8 )& 0xFFFFFF);
		break;
		
		case 2:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFFFFFF0000) | ((temp128.lo >> 16) & 0xFFFF);
		break;
		
		case 3:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFFFFFFFF00) | ((temp128.lo >> 24) & 0xFF);
		break;

		case 4:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 4 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = temp128.lo & 0xFFFFFFFF;
		break;
		
		case 5:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 4 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFFFF000000) | ((temp128.lo >> 8 )& 0xFFFFFF);
		break;
		
		case 6:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 4 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFFFFFF0000) | ((temp128.lo >> 16) & 0xFFFF);
		break;
		
		case 7:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 4 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFFFFFFFF00) | ((temp128.lo >> 24) & 0xFF);
		break;
		
		case 8:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 8 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = temp128.lo & 0xFFFFFFFF;
		break;
		
		case 9:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 8 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFFFF000000) | ((temp128.lo >> 8 )& 0xFFFFFF);
		break;
		
		case 10:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 8 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFFFFFF0000) | ((temp128.lo >> 16) & 0xFFFF);
		break;
		
		case 11:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 8 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFFFFFFFF00) | ((temp128.lo >> 24) & 0xFF);
		break;

		case 12:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 12 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = temp128.lo & 0xFFFFFFFF;
		break;
		
		case 13:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 12 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFFFF000000) | ((temp128.lo >> 8 )& 0xFFFFFF);
		break;
		
		case 14:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 12 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFFFFFF0000) | ((temp128.lo >> 16) & 0xFFFF);
		break;
		
		case 15:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 12 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFFFFFFFF00) | ((temp128.lo >> 24) & 0xFF);
		break;

		

	}


	INCREMENT_PC();
}


void    pLWR  () {		
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = LWR;



	PrefetchExtractbase();
	PrefetchExtractrt();
	PrefetchExtractoffset();

	sign_extendedh(MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset);
		
	
}




/*
Load Word Unsigned
To load a word from memory as an unsigned value
*/
void    LWU   () {
	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: lwu %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[CURRENT_RT],CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );

	//sign_extendedh(offset);
	MipsMemoryRead ( &MipsCpu.GPR[CURRENT_RT] , DATA_W ,  MipsCpu.GPR[CURRENT_BASE].lo , CURRENT_OFFSET ) ;

    INCREMENT_PC();
}

void    pLWU   () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = LWU;
	PrefetchExtractbase();
	PrefetchExtractrt();
	PrefetchExtractoffset();

	sign_extendedh(MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset);
	

}


/*
Move From HI Register
To copy the special purpose HI register to a GPR
*/
void   MFHI   () {

	if ( debug_op == 1 ) sprintf(debugTempText,"%8.8llx: mhi %s", MipsCpu.PC, debug_r5900_reg[CURRENT_RD]  );
	
	MipsCpu.GPR[CURRENT_RD].lo = MipsCpu.GPR[CPU_REG_HI].lo;

    INCREMENT_PC();
}

void   pMFHI   () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MFHI;
	PrefetchExtractrd();
	

}


/*
Move From LO Register
To copy the special purpose LO register to a GPR
*/
void   MFLO  () {		
	if ( debug_op == 1 ) sprintf(debugTempText,"%8.8llx: mflo %s", MipsCpu.PC, debug_r5900_reg[CURRENT_RD]  );
	
	MipsCpu.GPR[CURRENT_RD].lo = MipsCpu.GPR[CPU_REG_LO].lo;

    INCREMENT_PC();
}

void   pMFLO  () {		
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MFLO;
	PrefetchExtractrd();
	

}


/*
Move Conditional on Not Zero
To conditionally move a GPR after testing a GPR value
*/
void   MOVN() {
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: movn %s,%s,%s ", MipsCpu.PC, debug_r5900_reg[CURRENT_RD],debug_r5900_reg[CURRENT_RS] ,debug_r5900_reg[CURRENT_RT] );
	
	if(  MipsCpu.GPR[CURRENT_RT].lo != 0 ) MipsCpu.GPR[CURRENT_RD].lo = MipsCpu.GPR[CURRENT_RS].lo ;
		
	INCREMENT_PC();
}

void   pMOVN() {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MOVN;
	PrefetchExtractrs();
	PrefetchExtractrt();
	PrefetchExtractrd();
	

}


/*
Move Conditional on Zero
To conditionally move a GPR after testing a GPR value
*/
void   MOVZ() {		
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: movz %s,%s,%s", MipsCpu.PC, debug_r5900_reg[CURRENT_RD],debug_r5900_reg[CURRENT_RS] ,debug_r5900_reg[CURRENT_RT] );
	
	if(  MipsCpu.GPR[CURRENT_RT].lo == 0 ) MipsCpu.GPR[CURRENT_RD].lo = MipsCpu.GPR[CURRENT_RS].lo ;
		
	INCREMENT_PC();
}

void   pMOVZ() {		
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MOVZ;
	PrefetchExtractrs();
	PrefetchExtractrt();
	PrefetchExtractrd();
	

}


void   MTHI  () {	MipsCpu.halt = 1;}
void   pMTHI  () {		MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MTHI;
}

void   MTLO  () {	MipsCpu.halt = 1;}
void   pMTLO  () {		MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MTLO;
}


/*
Multiply Word
To multiply 32-bit signed integers
*/
void   MULT     () {
	_s64 temp;
	
	if ( debug_op == 1 ) sprintf(debugTempText,"%8.8llx: mult %s,%s,%s", MipsCpu.PC, debug_r5900_reg[CURRENT_RD] , debug_r5900_reg[CURRENT_RS],debug_r5900_reg[CURRENT_RT] );

    if ( IsSignExtended( MipsCpu.GPR[CURRENT_RS].lo ) & IsSignExtended( MipsCpu.GPR[CURRENT_RT].lo )   ) {   
		
		temp = (_s64)((_s32)(MipsCpu.GPR[CURRENT_RS].lo & 0xFFFFFFFF )) * ((_s32)( MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF ));
		
		MipsCpu.GPR[CPU_REG_LO].lo = temp & 0xFFFFFFFF;
		MipsCpu.GPR[CPU_REG_HI].lo = temp  >> 32 ;
		
		sign_extended(MipsCpu.GPR[CPU_REG_LO].lo);
		sign_extended(MipsCpu.GPR[CPU_REG_HI].lo);
		
		if ( CURRENT_RD != 0 ) MipsCpu.GPR[CURRENT_RD].lo = MipsCpu.GPR[CPU_REG_LO].lo ;

    }
	
	INCREMENT_PC();
}

void   pMULT     () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MULT;

	
	PrefetchExtractrs();
	PrefetchExtractrt();
	PrefetchExtractrd();			
	

}


void   MULTU() {		
    _u64 temp;
	
	if ( debug_op == 1 ) sprintf(debugTempText,"%8.8llx: multu %s,%s,%s", MipsCpu.PC, debug_r5900_reg[CURRENT_RD] , debug_r5900_reg[CURRENT_RS],debug_r5900_reg[CURRENT_RT] );

    if ( IsSignExtended( MipsCpu.GPR[CURRENT_RS].lo ) & IsSignExtended( MipsCpu.GPR[CURRENT_RT].lo )   ) {   
		
//		printf("MULTU : %lx * %lx",((_u32)(MipsCpu.GPR[rs].lo  )),((_u32)( MipsCpu.GPR[rt].lo  )) );
		
		temp = (_u64)((_u32)(MipsCpu.GPR[CURRENT_RS].lo & 0xFFFFFFFF )) * ((_u32)( MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF ));
		
//		printf(" = %llx\n",temp);
		
		MipsCpu.GPR[CPU_REG_LO].lo = temp & 0xFFFFFFFF;
		MipsCpu.GPR[CPU_REG_HI].lo = temp  >> 32 ;
		
//		sign_extended(MipsCpu.GPR[CPU_REG_LO].lo);
//		sign_extended(MipsCpu.GPR[CPU_REG_HI].lo);
		
		if ( CURRENT_RD != 0 ) MipsCpu.GPR[CURRENT_RD].lo = MipsCpu.GPR[CPU_REG_LO].lo ;

    }
	
	INCREMENT_PC();
}

void   pMULTU() {		
 	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MULTU;

	
	PrefetchExtractrs();
	PrefetchExtractrt();
	PrefetchExtractrd();			
	

}


void   NOR   () {	MipsCpu.halt = 1;}
void   pNOR   () {		MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = NOR;
}


/*
Or
To do a bitwise logical OR
*/
void   OR () {

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: or %s,%s,%s", MipsCpu.PC, debug_r5900_reg[CURRENT_RD],debug_r5900_reg[CURRENT_RS],debug_r5900_reg[CURRENT_RT] );

    MipsCpu.GPR[CURRENT_RD].lo = MipsCpu.GPR[CURRENT_RS].lo | MipsCpu.GPR[CURRENT_RT].lo;
 
    INCREMENT_PC();
}

void   pOR () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = OR;
	PrefetchExtractrt(); 
	PrefetchExtractrs();
	PrefetchExtractrd();

}


/*
Or Immediate
To do a bitwise logical OR with a constant
*/
void    ORI () {
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: ori	%s,%s,%4llx", MipsCpu.PC, debug_r5900_reg[CURRENT_RT],debug_r5900_reg[CURRENT_RS],CURRENT_IMM );
	
//	sign_extendedh(imm);	
	MipsCpu.GPR[CURRENT_RT].lo = MipsCpu.GPR[CURRENT_RS].lo | CURRENT_IMM;
	
	INCREMENT_PC();
}

void    pORI () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = ORI;
	PrefetchExtractrt(); 
	PrefetchExtractrs();
	PrefetchExtractimm();
	

}


void    PREF () {	MipsCpu.halt = 1;}
void    pPREF () {		MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = PREF;
}


/*
Store Byte
To store a byte to memory
*/
void    SB     () {
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sb %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[CURRENT_RT],CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );
	
	//sign_extendedh(offset);
	MipsMemoryWrite ( MipsCpu.GPR[CURRENT_RT] , DATA_B ,  MipsCpu.GPR[CURRENT_BASE].lo , CURRENT_OFFSET ) ;
	
	INCREMENT_PC();
}

void    pSB     () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = SB;
	PrefetchExtractbase();
	PrefetchExtractrt();
	PrefetchExtractoffset();

	sign_extendedh(MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset);
	

}


/*
Store Doubleword
To store a doubleword to memory
*/
void    SD() {
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sd %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[CURRENT_RT],CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );
	
	//sign_extendedh(offset);
	MipsMemoryWrite ( MipsCpu.GPR[CURRENT_RT] , DATA_DW ,  MipsCpu.GPR[CURRENT_BASE].lo , CURRENT_OFFSET ) ;
	
	INCREMENT_PC();
}


void    pSD() {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = SD;
	PrefetchExtractbase();
	PrefetchExtractrt();
	PrefetchExtractoffset();
	
	sign_extendedh(MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset);

	
}

/*
Store Doubleword Left
To store the most-significant part of a doubleword to an unaligned memory address
*/
void    SDL  () {

    _u128   temp128;
    _u64    ReadAdress;

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sdl %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[CURRENT_RT],CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );

	//sign_extendedh(offset);

	switch ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xF)
	{
		case 0:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			temp128.lo = ((_u64)MipsCpu.GPR[CURRENT_RT].lo >> 56)|(temp128.lo & 0xFFFFFFFFFFFFFF00);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		break;

		case 1:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			temp128.lo = ((_u64)MipsCpu.GPR[CURRENT_RT].lo >> 48)|(temp128.lo & 0xFFFFFFFFFFFF0000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		break;

		case 2:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			temp128.lo = ((_u64)MipsCpu.GPR[CURRENT_RT].lo >> 40)|(temp128.lo & 0xFFFFFFFFFF000000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		break;

		case 3:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			temp128.lo = ((_u64)MipsCpu.GPR[CURRENT_RT].lo >> 32)|(temp128.lo & 0xFFFFFFFF00000000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		break;

		case 4:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			temp128.lo = ((_u64)MipsCpu.GPR[CURRENT_RT].lo >> 24)|(temp128.lo & 0xFFFFFF0000000000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		break;

		case 5:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			temp128.lo = ((_u64)MipsCpu.GPR[CURRENT_RT].lo >> 16)|(temp128.lo & 0xFFFF000000000000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		break;

		case 6:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			temp128.lo = ((_u64)MipsCpu.GPR[CURRENT_RT].lo >> 8)|(temp128.lo & 0xFF00000000000000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		break;

		case 7:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
			MipsMemoryWrite ( MipsCpu.GPR[CURRENT_RT] , DATA_DW ,  ReadAdress , 0 ) ;
		break;

		case 8:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			temp128.lo = ((_u64)MipsCpu.GPR[CURRENT_RT].lo >> 56)|(temp128.lo & 0xFFFFFFFFFFFFFF00);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		break;

		case 9:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			temp128.lo = ((_u64)MipsCpu.GPR[CURRENT_RT].lo >> 48)|(temp128.lo & 0xFFFFFFFFFFFF0000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		break;

		case 10:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			temp128.lo = ((_u64)MipsCpu.GPR[CURRENT_RT].lo >> 40)|(temp128.lo & 0xFFFFFFFFFF000000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		break;

		case 11:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			temp128.lo = ((_u64)MipsCpu.GPR[CURRENT_RT].lo >> 32)|(temp128.lo & 0xFFFFFFFF00000000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		break;

		case 12:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			temp128.lo = ((_u64)MipsCpu.GPR[CURRENT_RT].lo >> 24)|(temp128.lo & 0xFFFFFF0000000000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		break;

		case 13:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			temp128.lo = ((_u64)MipsCpu.GPR[CURRENT_RT].lo >> 16)|(temp128.lo & 0xFFFF000000000000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		break;

		case 14:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			temp128.lo = ((_u64)MipsCpu.GPR[CURRENT_RT].lo >> 8)|(temp128.lo & 0xFF00000000000000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		break;

		case 15:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) + 8;
			MipsMemoryWrite ( MipsCpu.GPR[CURRENT_RT] , DATA_DW ,  ReadAdress , 0 ) ;
		break;
	}

	INCREMENT_PC();
}


void    pSDL  () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = SDL;


 
    PrefetchExtractbase();
	PrefetchExtractrt();
	PrefetchExtractoffset();

	sign_extendedh(MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset);
	

}

/*
Store Doubleword Right
To store the least-significant part of a doubleword to an unaligned memory address
*/

void    SDR  () {
    _u128   temp128;
    _u64    ReadAdress;

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sdr %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[CURRENT_RT],CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );

	//sign_extendedh(offset);

	switch ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xF)
	{
	case 0:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) ;
		MipsMemoryWrite ( MipsCpu.GPR[CURRENT_RT] , DATA_DW ,  ReadAdress , 0 ) ;
	break;

	case 1:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
		MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		temp128.lo = (MipsCpu.GPR[CURRENT_RT].lo << 8) | (temp128.lo & 0xFFFFFFFFFFFFFF00);
		MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
	break;

	case 2:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
		MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		temp128.lo = (MipsCpu.GPR[CURRENT_RT].lo << 16) | (temp128.lo & 0xFFFFFFFFFFFF0000);
		MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
	break;

	case 3:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
		MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		temp128.lo = (MipsCpu.GPR[CURRENT_RT].lo << 24) | (temp128.lo & 0xFFFFFFFFFF000000);
		MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
	break;

	case 4:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
		MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		temp128.lo = (MipsCpu.GPR[CURRENT_RT].lo << 32) | (temp128.lo & 0xFFFFFFFF00000000);
		MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
	break;

	case 5:	
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
		MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		temp128.lo = (MipsCpu.GPR[CURRENT_RT].lo << 40) | (temp128.lo & 0xFFFFFF0000000000);
		MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
	break;

	case 6:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
		MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		temp128.lo = (MipsCpu.GPR[CURRENT_RT].lo << 48) | (temp128.lo & 0xFFFF000000000000);
		MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
	break;

	case 7:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
		MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		temp128.lo = (MipsCpu.GPR[CURRENT_RT].lo << 56) | (temp128.lo & 0xFF00000000000000);
		MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
	break;
	
	case 8:
		ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) + 8;
		MipsMemoryWrite ( MipsCpu.GPR[CURRENT_RT] , DATA_DW ,  ReadAdress , 0 ) ;
	break;

	case 9:	
		ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) + 8;
		MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		temp128.lo = (MipsCpu.GPR[CURRENT_RT].lo << 8) | (temp128.lo & 0xFFFFFFFFFFFFFF00);
		MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
	break;
	
	case 10:
		ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) + 8;
		MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		temp128.lo = (MipsCpu.GPR[CURRENT_RT].lo << 16) | (temp128.lo & 0xFFFFFFFFFFFF0000);
		MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
	break;
	
	case 11:
		ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) + 8;
		MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		temp128.lo = (MipsCpu.GPR[CURRENT_RT].lo << 24) | (temp128.lo & 0xFFFFFFFFFF000000);
		MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
	break;

	case 12:
		ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) + 8;
		MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		temp128.lo = (MipsCpu.GPR[CURRENT_RT].lo << 32) | (temp128.lo & 0xFFFFFFFF00000000);
		MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
	break;

	case 13:
		ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) + 8;
		MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		temp128.lo = (MipsCpu.GPR[CURRENT_RT].lo << 40) | (temp128.lo & 0xFFFFFF0000000000);
		MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
	break;

	case 14:
		ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) + 8;
		MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		temp128.lo = (MipsCpu.GPR[CURRENT_RT].lo << 48) | (temp128.lo & 0xFFFF000000000000);
		MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
	break;

	case 15:
		ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ) + 8;
		MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		temp128.lo = (MipsCpu.GPR[CURRENT_RT].lo << 56) | (temp128.lo & 0xFF00000000000000);
		MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
	break;
	}

	INCREMENT_PC();
}


void    pSDR  () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = SDR;


	PrefetchExtractbase();
	PrefetchExtractrt();
	PrefetchExtractoffset();

	sign_extendedh(MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset);
	

}



/*
Store Halfword
To store a halfword to memory
*/
void    SH    () {
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sh %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[CURRENT_RT],CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );
	
	//sign_extendedh(offset);
	MipsMemoryWrite ( MipsCpu.GPR[CURRENT_RT] , DATA_HW ,  MipsCpu.GPR[CURRENT_BASE].lo , CURRENT_OFFSET ) ;
	
	INCREMENT_PC();
}

void    pSH    () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = SH;
	PrefetchExtractbase();
	PrefetchExtractrt();
	PrefetchExtractoffset();
	
	sign_extendedh(MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset);


}


/*
Shift Word Left Logical
To left-shift a word by a fixed number of bits
*/
void   SLL() {
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sll %s,%s,%ld", MipsCpu.PC, debug_r5900_reg[CURRENT_RD], debug_r5900_reg[CURRENT_RT], CURRENT_SA );
	
	
	MipsCpu.GPR[CURRENT_RD].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF) <<  CURRENT_SA;
	sign_extended(MipsCpu.GPR[CURRENT_RD].lo);
	
	INCREMENT_PC();
}

void   pSLL() {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = SLL;
	PrefetchExtractrt();
	PrefetchExtractrd();
	PrefetchExtractsa();
	

}


/*
Shift Word Left Logical Variable
To left-shift a word by a variable number of bits
*/
void   SLLV () {		
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sllv %s,%s,%s", MipsCpu.PC, debug_r5900_reg[CURRENT_RD], debug_r5900_reg[CURRENT_RT], debug_r5900_reg[CURRENT_RS] );
 
	MipsCpu.GPR[CURRENT_RD].lo = ( MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF ) <<  ( MipsCpu.GPR[CURRENT_RS].lo & 0x1F);	
	sign_extended(MipsCpu.GPR[CURRENT_RD].lo);

	INCREMENT_PC();
}

void   pSLLV () {		
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = SLLV;
	PrefetchExtractrt();
	PrefetchExtractrd();
	PrefetchExtractrs();
	

}


/*
Set on Less Than
To record the result of a less-than comparison
*/
void   SLT () {
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: slt %s,%s,%s", MipsCpu.PC, debug_r5900_reg[CURRENT_RD], debug_r5900_reg[CURRENT_RS], debug_r5900_reg[CURRENT_RT]);
	
	if  ((_s64) MipsCpu.GPR[CURRENT_RS].lo < (_s64) MipsCpu.GPR[CURRENT_RT].lo ) MipsCpu.GPR[CURRENT_RD].lo = 1; 
	else 	MipsCpu.GPR[CURRENT_RD].lo = 0; 
	
	INCREMENT_PC();
}

void   pSLT () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = SLT;
	PrefetchExtractrs();
	PrefetchExtractrt();
	PrefetchExtractrd();
	
}


/*
Set on Less Than Immediate
To record the result of a less-than comparison with a constant
*/
void    SLTI() {
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: slti %s,%s,%4llx", MipsCpu.PC, debug_r5900_reg[CURRENT_RT],debug_r5900_reg[CURRENT_RS],CURRENT_IMM );
	//sign_extendedh(imm);
	
	if ( (_s64)MipsCpu.GPR[CURRENT_RS].lo < (_s64) CURRENT_IMM ) 	MipsCpu.GPR[CURRENT_RT].lo = 1;
	else MipsCpu.GPR[CURRENT_RT].lo = 0;
	
	INCREMENT_PC();
}

void    pSLTI() {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = SLTI;
	PrefetchExtractrt(); 
	PrefetchExtractrs();
	PrefetchExtractimm();

	sign_extendedh(MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->imm);
	
}



/*
Set on Less Than Immediate Unsigned
To record the result of an unsigned less-than comparison with a constant
*/
void    SLTIU() {
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sltiu %s,%s,%8llx", MipsCpu.PC, debug_r5900_reg[CURRENT_RD], debug_r5900_reg[CURRENT_RS], CURRENT_IMM);
	
    //sign_extendedh(imm);
	
	if  ( (_u64)MipsCpu.GPR[CURRENT_RS].lo < (_u64)CURRENT_IMM ) 	MipsCpu.GPR[CURRENT_RT].lo = 1; 
	else 	MipsCpu.GPR[CURRENT_RT].lo = 0; 
	
	INCREMENT_PC();
}

void    pSLTIU() {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = SLTIU;
	PrefetchExtractrs();
	PrefetchExtractrt();
	PrefetchExtractimm();
	
	sign_extendedh(MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->imm);
	

}



/*
Set on Less Than Unsigned
To record the result of an unsigned less-than comparison
*/
void   SLTU   () {
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sltu %s,%s,%s", MipsCpu.PC, debug_r5900_reg[CURRENT_RD], debug_r5900_reg[CURRENT_RS], debug_r5900_reg[CURRENT_RT]);
	
	if  ( (_u64) MipsCpu.GPR[CURRENT_RS].lo < (_u64)MipsCpu.GPR[CURRENT_RT].lo ) 	MipsCpu.GPR[CURRENT_RD].lo = 1; 
	else 	MipsCpu.GPR[CURRENT_RD].lo = 0; 
	
	INCREMENT_PC();
}

void   pSLTU   () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = SLTU;
	PrefetchExtractrs();
	PrefetchExtractrt();
	PrefetchExtractrd();
	
	
}

/*
Shift Word Right Arithmetic
To execute an arithmetic right-shift of a word by a fixed number of bits
*/
void   SRA  () {
	_u64 temp;
	_u8 boucle;
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sra %s,%s,%ld", MipsCpu.PC, debug_r5900_reg[CURRENT_RD], debug_r5900_reg[CURRENT_RT], CURRENT_SA );
	
	if ( IsSignExtended( MipsCpu.GPR[CURRENT_RT].lo )   ) { 
	
		if ((MipsCpu.GPR[CURRENT_RT].lo & SIGN_BIT_MASK) == SIGN_BIT_MASK) {
			
			temp = MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF;
			for ( boucle = 0 ; boucle < CURRENT_SA ; boucle++ ) temp = ( temp >> 1 ) | SIGN_BIT_MASK ;
			MipsCpu.GPR[CURRENT_RD].lo = temp;
			
		} else 	MipsCpu.GPR[CURRENT_RD].lo =  MipsCpu.GPR[CURRENT_RT].lo >>  CURRENT_SA  ; 
	
	}
      		
	INCREMENT_PC();
}

void   pSRA  () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = SRA;
	
	PrefetchExtractrt();
	PrefetchExtractrd();
	PrefetchExtractsa();
	

}

/*
Shift Word Right Arithmetic Variable
To execute an arithmetic right-shift of a word by a variable number of bits
*/
void   SRAV () {		
	_u64 temp;
	_u8 boucle;

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: srav %s,%s,%s", MipsCpu.PC, debug_r5900_reg[CURRENT_RD], debug_r5900_reg[CURRENT_RT], debug_r5900_reg[CURRENT_RS] );
	
	if ( IsSignExtended( MipsCpu.GPR[CURRENT_RT].lo )   ) { 
		
		if ((MipsCpu.GPR[CURRENT_RT].lo & SIGN_BIT_MASK) == SIGN_BIT_MASK) {
			
			temp = MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF;
			for ( boucle = 0 ; boucle < ( MipsCpu.GPR[CURRENT_RS].lo & 0x1F) ; boucle++ ) temp = ( temp >> 1 ) | SIGN_BIT_MASK ;
			MipsCpu.GPR[CURRENT_RD].lo = temp;
			
		} else 	MipsCpu.GPR[CURRENT_RD].lo =  MipsCpu.GPR[CURRENT_RT].lo >>  ( MipsCpu.GPR[CURRENT_RS].lo & 0x1F)  ; 
	
	}
	
	INCREMENT_PC();
}

void   pSRAV () {		
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = SRAV;
	
	PrefetchExtractrt();
	PrefetchExtractrd();
	PrefetchExtractrs();
	

}


/*
Shift Word Right Logical
To execute a logical right-shift of a word by a fixed number of bits
*/
void   SRL   () {
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: srl %s,%s,%ld", MipsCpu.PC, debug_r5900_reg[CURRENT_RD], debug_r5900_reg[CURRENT_RT], CURRENT_SA );
	
	if ( IsSignExtended( MipsCpu.GPR[CURRENT_RT].lo )   ) { 
		
		MipsCpu.GPR[CURRENT_RD].lo = ( MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF )  >>  CURRENT_SA;	
		sign_extended(MipsCpu.GPR[CURRENT_RD].lo);
	}	
	INCREMENT_PC();
}

void   pSRL   () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = SRL;
	PrefetchExtractrt();
	PrefetchExtractrd();
	PrefetchExtractsa();
	

}


/*
Shift Word Right Logical Variable
To execute a logical right-shift of a word by a variable number of bits
*/
void   SRLV () {	
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: srlv %s,%s,%s", MipsCpu.PC, debug_r5900_reg[CURRENT_RD], debug_r5900_reg[CURRENT_RT], debug_r5900_reg[CURRENT_RS] );
	
	if ( IsSignExtended( MipsCpu.GPR[CURRENT_RT].lo )   ) { 
		
		MipsCpu.GPR[CURRENT_RD].lo = ( MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF )  >>  ( MipsCpu.GPR[CURRENT_RS].lo & 0x1F);	
		sign_extended(MipsCpu.GPR[CURRENT_RD].lo);
	
	}
	
	INCREMENT_PC();
}

void   pSRLV () {	
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = SRLV;
	PrefetchExtractrt();
	PrefetchExtractrd();
	PrefetchExtractrs();
	

}


/*
Subtract Word
To subtract 32-bit integers. If overflow occurs, then trap
*/
void   SUB () {
	_u64 temp;

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sub	%s,%s,%s", MipsCpu.PC, debug_r5900_reg[CURRENT_RD],debug_r5900_reg[CURRENT_RS],debug_r5900_reg[CURRENT_RT] );

	if ( IsSignExtended( MipsCpu.GPR[CURRENT_RS].lo )  & IsSignExtended( MipsCpu.GPR[CURRENT_RT].lo ) ) { 
	
		temp = MipsCpu.GPR[CURRENT_RS].lo - MipsCpu.GPR[CURRENT_RT].lo;	
		
		if ( IsNot32bitIntegerOverflow ( temp ) ) {
			sign_extended(temp);
			MipsCpu.GPR[CURRENT_RD].lo = temp;
		}
		else {
			//SignalExeption(IntegerOverflow)
		}
	}
	
	INCREMENT_PC();
}

void   pSUB () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = SUB;

	
	PrefetchExtractrt(); 
	PrefetchExtractrs();
	PrefetchExtractrd();
}


/*
Subtract Unsigned Word 
To subtract 32-bit integers
*/
void   SUBU   () {
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: subu %s,%s,%s", MipsCpu.PC, debug_r5900_reg[CURRENT_RD],debug_r5900_reg[CURRENT_RS],debug_r5900_reg[CURRENT_RT] );

	if ( IsSignExtended( MipsCpu.GPR[CURRENT_RS].lo )  & IsSignExtended( MipsCpu.GPR[CURRENT_RT].lo ) ) { 
		
		MipsCpu.GPR[CURRENT_RD].lo = MipsCpu.GPR[CURRENT_RS].lo - MipsCpu.GPR[CURRENT_RT].lo;
		sign_extended(MipsCpu.GPR[CURRENT_RD].lo);
	
	}
	
	INCREMENT_PC();
}

void   pSUBU   () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = SUBU;
	PrefetchExtractrt(); 
	PrefetchExtractrs();
	PrefetchExtractrd();
	

}


/*
Store Word
To store a word to memory
*/
void    SW   () {
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sd %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[CURRENT_RT],CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );
	
	//sign_extendedh(offset);
	MipsMemoryWrite ( MipsCpu.GPR[CURRENT_RT] , DATA_W ,  MipsCpu.GPR[CURRENT_BASE].lo , CURRENT_OFFSET ) ;

    INCREMENT_PC();
}

void    pSW   () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = SW;
	PrefetchExtractbase();
	PrefetchExtractrt();
	PrefetchExtractoffset();
	
	sign_extendedh(MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset);


}


/*
Store Word Left
To store the most-significant part of a word to an unaligned memory address
MIPS32
*/
void    SWL2 () {		
	_u128	temp128;
	_u64	temp64;

	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: swl %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[CURRENT_RT],CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );

    printf("SWL\n");

	sign_extendedh(CURRENT_OFFSET);
	
	temp64 = MipsCpu.GPR[CURRENT_RT].lo;
	temp64 = temp64 & 0xFFFF0000;
	temp128.lo = temp64;

	MipsMemoryWrite ( temp128 , DATA_W ,  MipsCpu.GPR[CURRENT_BASE].lo , CURRENT_OFFSET ) ;

    INCREMENT_PC();
}

void    SWL  () {

    _u128   temp128;
    _u64    ReadAdress;

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sdl %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[CURRENT_RT],CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );

	//sign_extendedh(offset);

	switch ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xF)
	{
		case 0:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			temp128.lo = ( (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF) >> 24)|(temp128.lo & 0xFFFFFFFFFFFFFF00);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		break;

		case 1:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF) >> 16)|(temp128.lo & 0xFFFFFFFFFFFF0000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		break;

		case 2:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF) >> 8)|(temp128.lo & 0xFFFFFFFFFF000000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		break;

		case 3:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF) )|(temp128.lo & 0xFFFFFFFF00000000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		break;

		case 4:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 4 ) ;
			temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF) >> 24)|(temp128.lo & 0xFFFFFFFFFFFFFF00);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 4 ) ;
		break;

		case 5:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 4 ) ;
			temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF) >> 16)|(temp128.lo & 0xFFFFFFFFFFFF0000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 4 ) ;
		break;

		case 6:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 4 ) ;
			temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF) >> 8)|(temp128.lo & 0xFFFFFFFFFF000000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 4 ) ;
		break;

		case 7:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 4 ) ;
			temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF) )|(temp128.lo & 0xFFFFFFFF00000000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 4 ) ;
		break;

		case 8:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 8 ) ;
			temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF) >> 24)|(temp128.lo & 0xFFFFFFFFFFFFFF00);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 8 ) ;
		break;

		case 9:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 8 ) ;
			temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF) >> 16)|(temp128.lo & 0xFFFFFFFFFFFF0000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 8 ) ;
		break;

		case 10:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 8 ) ;
			temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF) >> 8)|(temp128.lo & 0xFFFFFFFFFF000000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 8 ) ;
		break;

		case 11:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 8 ) ;
			temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF) )|(temp128.lo & 0xFFFFFFFF00000000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 8 ) ;
		break;

		case 12:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 12 ) ;
			temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF) >> 24)|(temp128.lo & 0xFFFFFFFFFFFFFF00);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 12 ) ;
		break;

		case 13:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 12 ) ;
			temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF) >> 16)|(temp128.lo & 0xFFFFFFFFFFFF0000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 12 ) ;
		break;

		case 14:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 12 ) ;
			temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF) >> 8)|(temp128.lo & 0xFFFFFFFFFF000000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 12 ) ;
		break;

		case 15:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 12 ) ;
			temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF) )|(temp128.lo & 0xFFFFFFFF00000000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 12 ) ;
		break;


	}

	INCREMENT_PC();
}


void    pSWL () {		
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = SWL;

	PrefetchExtractbase();
	PrefetchExtractrt();
	PrefetchExtractoffset();
	
	sign_extendedh(MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset);	

}


/*
Store Word Right
To store the least-significant part of a word to an unaligned memory address
MIPS32
*/
void    SWR2  () {		
	_u128	temp128;
	_u64	temp64;

    printf("SWR\n");	

	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: swl %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[CURRENT_RT],CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );

	sign_extendedh(CURRENT_OFFSET);
	
	temp64 = MipsCpu.GPR[CURRENT_RT].lo;
	temp64 = temp64 & 0xFFFF;
	temp128.lo = temp64;

	MipsMemoryWrite ( temp128 , DATA_W ,  MipsCpu.GPR[CURRENT_BASE].lo , CURRENT_OFFSET ) ;

    INCREMENT_PC();
}

void    SWR  () {
    _u128   temp128;
    _u64    ReadAdress;

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sdr %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[CURRENT_RT],CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );

	//sign_extendedh(offset);

	switch ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xF)
	{
	case 0:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) ;
		MipsMemoryWrite ( MipsCpu.GPR[CURRENT_RT] , DATA_W ,  ReadAdress , 0 ) ;
	break;

	case 1:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
		MipsMemoryRead ( &temp128 , DATA_W ,  ReadAdress , 0 ) ;
		temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo << 8) & 0xFFFFFF00) | (temp128.lo & 0x000000FF);
		MipsMemoryWrite ( temp128 , DATA_W ,  ReadAdress , 0 ) ;
	break;

	case 2:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
		MipsMemoryRead ( &temp128 , DATA_W ,  ReadAdress , 0 ) ;
		temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo << 16) & 0xFFFF0000) | (temp128.lo & 0x0000FFFF);
		MipsMemoryWrite ( temp128 , DATA_W ,  ReadAdress , 0 ) ;
	break;

	case 3:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
		MipsMemoryRead ( &temp128 , DATA_W ,  ReadAdress , 0 ) ;
		temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo << 24) & 0xFF000000) | (temp128.lo & 0x00FFFFFF);
		MipsMemoryWrite ( temp128 , DATA_W ,  ReadAdress , 0 ) ;
	break;

	case 4:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) ;
		MipsMemoryWrite ( MipsCpu.GPR[CURRENT_RT] , DATA_W ,  ReadAdress , 4 ) ;
	break;

	case 5:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
		MipsMemoryRead ( &temp128 , DATA_W ,  ReadAdress , 4 ) ;
		temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo << 8) & 0xFFFFFF00) | (temp128.lo & 0x000000FF);
		MipsMemoryWrite ( temp128 , DATA_W ,  ReadAdress , 4 ) ;
	break;

	case 6:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
		MipsMemoryRead ( &temp128 , DATA_W ,  ReadAdress , 4 ) ;
		temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo << 16) & 0xFFFF0000) | (temp128.lo & 0x0000FFFF);
		MipsMemoryWrite ( temp128 , DATA_W ,  ReadAdress , 4 ) ;
	break;

	case 7:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
		MipsMemoryRead ( &temp128 , DATA_W ,  ReadAdress , 4 ) ;
		temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo << 24) & 0xFF000000) | (temp128.lo & 0x00FFFFFF);
		MipsMemoryWrite ( temp128 , DATA_W ,  ReadAdress , 4 ) ;
	break;

	case 8:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) ;
		MipsMemoryWrite ( MipsCpu.GPR[CURRENT_RT] , DATA_W ,  ReadAdress , 8 ) ;
	break;

	case 9:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
		MipsMemoryRead ( &temp128 , DATA_W ,  ReadAdress , 8 ) ;
		temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo << 8) & 0xFFFFFF00) | (temp128.lo & 0x000000FF);
		MipsMemoryWrite ( temp128 , DATA_W ,  ReadAdress , 8 ) ;
	break;

	case 10:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
		MipsMemoryRead ( &temp128 , DATA_W ,  ReadAdress , 8 ) ;
		temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo << 16) & 0xFFFF0000) | (temp128.lo & 0x0000FFFF);
		MipsMemoryWrite ( temp128 , DATA_W ,  ReadAdress , 8 ) ;
	break;

	case 11:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
		MipsMemoryRead ( &temp128 , DATA_W ,  ReadAdress , 8 ) ;
		temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo << 24) & 0xFF000000) | (temp128.lo & 0x00FFFFFF);
		MipsMemoryWrite ( temp128 , DATA_W ,  ReadAdress , 8 ) ;
	break;

	case 12:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) ;
		MipsMemoryWrite ( MipsCpu.GPR[CURRENT_RT] , DATA_W ,  ReadAdress , 12 ) ;
	break;

	case 13:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
		MipsMemoryRead ( &temp128 , DATA_W ,  ReadAdress , 12 ) ;
		temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo << 8) & 0xFFFFFF00) | (temp128.lo & 0x000000FF);
		MipsMemoryWrite ( temp128 , DATA_W ,  ReadAdress , 12 ) ;
	break;

	case 14:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
		MipsMemoryRead ( &temp128 , DATA_W ,  ReadAdress , 12 ) ;
		temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo << 16) & 0xFFFF0000) | (temp128.lo & 0x0000FFFF);
		MipsMemoryWrite ( temp128 , DATA_W ,  ReadAdress , 2 ) ;
	break;

	case 15:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET) & 0xFFFFFFF0 ;
		MipsMemoryRead ( &temp128 , DATA_W ,  ReadAdress , 12 ) ;
		temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo << 24) & 0xFF000000) | (temp128.lo & 0x00FFFFFF);
		MipsMemoryWrite ( temp128 , DATA_W ,  ReadAdress , 12 ) ;
	break;

	

	}

	INCREMENT_PC();
}



void    pSWR  () {		
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = SWR;

	PrefetchExtractbase();
	PrefetchExtractrt();
	PrefetchExtractoffset();

	sign_extendedh(MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset);

}



/*
Synchronize Shared Memory
To order loads and stores.
*/
void   SYNC  () {

    if ( debug_op == 1 ) sprintf(debugTempText,"%8.8llx: SYNC", MipsCpu.PC);
	
    INCREMENT_PC();
    
}
void   pSYNC  () {	
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = SYNC;
 }


/*
System Call
To cause a System Call exception
*/
void   SYSCALL() {
//	if ( debug_print_syscall == 1 )	printf("%8llx : %s ( %x h) \n", MipsCpu.PC , debug_syscall[(MipsCpu.GPR[3].lo & 0xFF)],(_u8) (MipsCpu.GPR[3].lo & 0xFF));
	syscall_fct[(MipsCpu.GPR[3].lo & 0xFF)]();
}

void   pSYSCALL() {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = SYSCALL;
//	if ( debug_print_syscall == 1 )	printf("%8llx : %s ( %x h) \n", MipsCpu.PC , debug_syscall[(MipsCpu.GPR[3].lo & 0xFF)],(_u8) (MipsCpu.GPR[3].lo & 0xFF));
//	syscall_fct[(MipsCpu.GPR[3].lo & 0xFF)]();
}


void   TEQ() 	{	MipsCpu.halt = 1;}
void   pTEQ() 	{		MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = TEQ;
}

void   TEQI() 	{	MipsCpu.halt = 1;}
void   pTEQI() 	{		MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = TEQI;
}


void   TGE() 	{	MipsCpu.halt = 1;}
void   pTGE() 	{		MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = TGE;
}

void   TGEI() 	{	MipsCpu.halt = 1;}
void   pTGEI() 	{		MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = TGEI;
}

void   TGEIU() 	{	MipsCpu.halt = 1;}
void   pTGEIU() 	{		MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = TGEIU;
}

void   TGEU() 	{	MipsCpu.halt = 1;}
void   pTGEU() 	{		MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = TGEU;
}

void   TLT() 	{	MipsCpu.halt = 1;}
void   pTLT() 	{		MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = TLT;
}

void   TLTI() 	{	MipsCpu.halt = 1;}
void   pTLTI() 	{	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = TLTI;
	}

void   TLTIU() 	{	MipsCpu.halt = 1;}
void   pTLTIU() 	{	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = TLTIU;
	}

void   TLTU() 	{	MipsCpu.halt = 1;}
void   pTLTU() 	{	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = TLTU;
	}

void   TNE() 	{	MipsCpu.halt = 1;}
void   pTNE() 	{	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = TNE;
}

void   TNEI() 	{	MipsCpu.halt = 1;}
void   pTNEI() 	{	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = TNEI;
	}

/*
Xor
To do a bitwise logical XOR
*/
void   XOR  () {

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: or %s,%s,%s", MipsCpu.PC, debug_r5900_reg[CURRENT_RD],debug_r5900_reg[CURRENT_RS],debug_r5900_reg[CURRENT_RT] );
	
	MipsCpu.GPR[CURRENT_RD].lo = MipsCpu.GPR[CURRENT_RS].lo ^ MipsCpu.GPR[CURRENT_RT].lo;
	
	INCREMENT_PC();
}

void   pXOR  () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = XOR;
	PrefetchExtractrt(); 
	PrefetchExtractrs();
	PrefetchExtractrd();

}


/*
Exclusive OR Immediate
To do a bitwise logical Exclusive OR with a constant
*/
void    XORI () {

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: xori %s,%s,%4llx", MipsCpu.PC, debug_r5900_reg[CURRENT_RT],debug_r5900_reg[CURRENT_RS],CURRENT_IMM );
	
	MipsCpu.GPR[CURRENT_RT].lo = MipsCpu.GPR[CURRENT_RS].lo ^ CURRENT_IMM;
	
	INCREMENT_PC();
}

void    pXORI () {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = XORI;
	PrefetchExtractrt(); 
	PrefetchExtractrs();
	PrefetchExtractimm();

}



/*****************************************************************************************************************
									COP0 INSTRUCTION SET
*****************************************************************************************************************/

/*
Move from Coprocessor 0
To move the contents of a coprocessor 0 register to a general register.
MIPS32
*/
void   	MF0() {


	if ( debug_op == 1 ) sprintf(debugTempText,"%8.8llx: mfc0 %s,%s,%ld", MipsCpu.PC, debug_r5900_reg[CURRENT_RT],debug_r5900_cop0_reg[CURRENT_RD],CURRENT_SEL );
	
	MipsCpu.GPR[CURRENT_RT].lo = MipsCpu.CPR0[CURRENT_RD][CURRENT_SEL];
//	sign_extended(MipsCpu.GPR[rt].lo);
	

   INCREMENT_PC();
	
}
void   	pMF0() {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MF0;

	PrefetchExtractrt();
	PrefetchExtractrd();
	PrefetchExtractsel();



	
}


void   	MT0() 	{	MipsCpu.halt = 1;}
void   	pMT0() 	{	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MT0;
	}

void	C0() 	{	cop0c0_opc[(MipsCpu.opcode&0x3F)](); }
void	pC0() 	{	pcop0c0_opc[((MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->opcode)&0x3F)]();
	 }

void 	BC0()	{	MipsCpu.halt = 1;}
void 	pBC0()	{	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = BC0;
	}


void	TLBR() 	{	MipsCpu.halt = 1;}
void	pTLBR() 	{	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = TLBR;
	}

void	TLBWI() {	MipsCpu.halt = 1;}
void	pTLBWI() {	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = TLBWI;
	}

void	TLBWR() {	MipsCpu.halt = 1;}
void	pTLBWR() {	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = TLBWR;
	}

void	TLBP() 	{	MipsCpu.halt = 1;}	
void	pTLBP() 	{	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = TLBP;
	}

void	ERET() 	{	MipsCpu.halt = 1;}
void	pERET() 	{	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = ERET;
	}





void 	EI() 	{
	MipsCpu.CPR0[12][0]	=	MipsCpu.CPR0[12][0]  | 0x00010000;
	INCREMENT_PC();
}

void 	pEI() 	{	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = EI;


}


void 	DI() 	{
	MipsCpu.CPR0[12][0]	=	MipsCpu.CPR0[12][0]  & 0xFFFEFFFF;
	INCREMENT_PC();
}

void 	pDI() 	{	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = DI;


}



/*****************************************************************************************************************
									COP1 INSTRUCTION SET
*****************************************************************************************************************/





void	BC1()	{ cop1_bc1_opc[(MipsCpu.opcode>>16)&0x3](); }
void	pBC1()	{ pcop1_bc1_opc[((MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->opcode)>>16)&0x3](); }	 		

void	COP1_S(){ cop1_s_opc[MipsCpu.opcode&0x3F]();   };
void	pCOP1_S(){ pcop1_s_opc[(MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->opcode)&0x3F]();   };

void	COP1_W(){ cop1_W_L_opc[MipsCpu.opcode&0x3F]();	}
void	pCOP1_W(){ pcop1_W_L_opc[(MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->opcode)&0x3F]();	}


void	ABS_S()		{ MipsCpu.halt = 1; } 	
void	pABS_S()		{	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = ABS_S;
  } 

/*
Floating Point Add
To add floating point values
*/
void	ADD_S()		{ 

 	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: add_s %s,%s,%s", MipsCpu.PC, debug_r5900_cop1_reg[CURRENT_FD],debug_r5900_cop1_reg[CURRENT_FS] ,debug_r5900_cop1_reg[CURRENT_FT]);
  
          
    *MipsCpu.FPR1_p_float[CURRENT_FD] = (float)((float)(*MipsCpu.FPR1_p_float[CURRENT_FS]) + (float)(*MipsCpu.FPR1_p_float[CURRENT_FT]));
    
 
          
                
	INCREMENT_PC();
} 

void	pADD_S()		{ 
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = ADD_S;

 
    PrefetchExtractfd();
	PrefetchExtractfs();
	PrefetchExtractft();
	
	} 		

/*
*/
void 	ADDA_S()	{ MipsCpu.halt = 1; } 
void 	pADDA_S()	{ 	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = ADDA_S;
 } 

/*
Branch on FP False
To branch according to the result of the immediately preceding floating-point comparaison operation.
*/
void	BC1F()		{ 	
	_u64	target_branch;
	_u32	condition;
	
	
	//offset = offset << 2;
	//sign_extendedh(offset);
	target_branch = MipsCpu.PC /*+ 4*/ +  CURRENT_OFFSET ;
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: bc1f %lx,%4llx", MipsCpu.PC, CURRENT_CCB,target_branch );

	condition = MipsCpu.CP1[CP1_FCSR] & CP1_FCSR_C_MASK;

	if ( condition == 0 ) {
		
		if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );
		
		INCREMENT_PC();
		MipsUpdateCpu();
		
		MipsCpu.PC = target_branch;
		
	}
	else INCREMENT_PC(); 
} 

void	pBC1F()		{ 	
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = BC1F;
	
	PrefetchExtractoffset();

	MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset = MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset << 2;
	sign_extendedh( MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset );
	MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset = MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset + 4;
	

} 

/*
*/

void	BC1FL()		{ MipsCpu.halt = 1; }
void	pBC1FL()		{ 	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = BC1FL;
 }

/*
Branch on FP True
To test an FP condition code and do a PC-relative conditional branch
*/
void	BC1T()		{ 	
	_u64	target_branch;
	_u32	condition;

	//offset = offset << 2;
	//sign_extendedh(offset);
	target_branch = MipsCpu.PC /*+ 4*/ +  CURRENT_OFFSET ;
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: bc1t %lx,%4llx", MipsCpu.PC, CURRENT_CCB,target_branch );

	condition = MipsCpu.CP1[CP1_FCSR] & CP1_FCSR_C_MASK;

	if ( condition == CP1_FCSR_C_MASK ) {
		
		if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );
		
		INCREMENT_PC();
		MipsUpdateCpu();
		
		MipsCpu.PC = target_branch;
	}
	else INCREMENT_PC();
} 

void	pBC1T()		{ 	
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = BC1T;
	
	PrefetchExtractoffset();

	MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset = MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset << 2;
	sign_extendedh( MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset );
	MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset = MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset + 4;

	

} 



/*
Branch on FP True Likely
To test an FP condition code and do a PC-relative conditional branch; execute the instruction in the delay slot only if
the branch is taken.
*/
void	BC1TL()		{ 	
	_u64	target_branch;
	_u32	condition;
	
	
	//offset = offset << 2;
	//sign_extendedh(offset);
	target_branch = MipsCpu.PC /*+ 4*/ +  CURRENT_OFFSET ;
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: bc1tl %lx,%4llx", MipsCpu.PC, CURRENT_CCB,target_branch );

	condition = MipsCpu.CP1[CP1_FCSR] & CP1_FCSR_C_MASK;
	
	if ( condition == CP1_FCSR_C_MASK ) {
		
		if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );
		
		INCREMENT_PC();
		MipsUpdateCpu();
		
		MipsCpu.PC = target_branch;
	}
	else MipsCpu.PC = MipsCpu.PC + 8; 
}

void	pBC1TL()		{ 	
		MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = BC1TL;

	PrefetchExtractoffset();

	MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset = MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset << 2;
	sign_extendedh( MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset );
	MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset = MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset + 4;

	

}


/*
Floating Point Compare
To compare single-precision floating-point values and record the result in flag C
*/
void	C_EQ_S()	{ 
	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: c_eq_s %lx,%s,%s", MipsCpu.PC, CURRENT_CC, debug_r5900_cop1_reg[CURRENT_FS],debug_r5900_cop1_reg[CURRENT_FT] );


	if ( (float)(*MipsCpu.FPR1_p_float[CURRENT_FS]) == (float) (*MipsCpu.FPR1_p_float[CURRENT_FT])  ) 
		MipsCpu.CP1[CP1_FCSR] = MipsCpu.CP1[CP1_FCSR] | CP1_FCSR_C_MASK;
	else  MipsCpu.CP1[CP1_FCSR] = MipsCpu.CP1[CP1_FCSR] & CP1_FCSR_C_NOT_MASK;
 
  	INCREMENT_PC();
}	

void	pC_EQ_S()	{ 
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = C_EQ_S;
	PrefetchExtractfs();
	PrefetchExtractft();
	

}		


void	C_F_S()		{ MipsCpu.halt = 1; } 		
void	pC_F_S()		{	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = C_F_S;
  } 	

void	C_OLE_S()	{ MipsCpu.halt = 1; }	
void	pC_OLE_S()	{ 	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = C_OLE_S;
 }	

/*
Floating Point Compare
To compare single-precision floating-point values and record the result in flag C
*/
void	C_OLT_S()	{ 	
	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: c_olt_s %lx,%s,%s", MipsCpu.PC, CURRENT_CC, debug_r5900_cop1_reg[CURRENT_FS],debug_r5900_cop1_reg[CURRENT_FT] );

	if ( (float)(*MipsCpu.FPR1_p_float[CURRENT_FS]) < (float)(*MipsCpu.FPR1_p_float[CURRENT_FT])  )
        MipsCpu.CP1[CP1_FCSR] = MipsCpu.CP1[CP1_FCSR] | CP1_FCSR_C_MASK;
	else MipsCpu.CP1[CP1_FCSR] = MipsCpu.CP1[CP1_FCSR] & CP1_FCSR_C_NOT_MASK;

	INCREMENT_PC();
}	

void	pC_OLT_S()	{ 	
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = C_OLT_S;
	PrefetchExtractfs();
	PrefetchExtractft();

}	

void	CFC1()		{ MipsCpu.halt = 1; }		
void	pCFC1()		{	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = CFC1;
 }

void	CTC1()		{ MipsCpu.halt = 1; }	
void	pCTC1()		{ 	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = CTC1;
 }

/*
Fixed-point Convert to Single Floating Point
To convert a 32 bit signed integer to a single-precision floating-point value
*/
void	CVTS_W()	{ 
	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: cvt_w_s	%s,%s", MipsCpu.PC, debug_r5900_cop1_reg[CURRENT_FS],debug_r5900_cop1_reg[CURRENT_FD] );

	*MipsCpu.FPR1_p_float[CURRENT_FD] = MipsCpu.FPR1[CURRENT_FS].lo;
    
 
  	INCREMENT_PC();
}

void	pCVTS_W()	{ 
 	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = CVTS_W;
   PrefetchExtractfs();
	PrefetchExtractfd();
	

}


/*
Floating Point Convert to Word Fixed Point
To convert a 32 bit signed integer to a single-precision fixed-point value
MIPS32
*/
void	CVT_W_S()	{ 
    _s16    f_exp;
	
    if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: cvt_w_s	%s, %s", MipsCpu.PC, debug_r5900_cop1_reg[CURRENT_FD],debug_r5900_cop1_reg[CURRENT_FS] );
       
    f_exp = (_u16)( (MipsCpu.FPR1[CURRENT_FS].lo >> 23) & 0xFF) ;
     
    if ( (_u16) f_exp <= 0x9D ) {
      MipsCpu.FPR1[CURRENT_FD].lo =*MipsCpu.FPR1_p_float[CURRENT_FS];
    }
    else {
        if ( (MipsCpu.FPR1[CURRENT_FS].lo & SIGN_BIT_MASK ) == 0 ) MipsCpu.FPR1[CURRENT_FD].lo = 0x7FFFFFFF;
        else MipsCpu.FPR1[CURRENT_FD].lo = 0x80000000;
   }



   INCREMENT_PC();
}	

void	pCVT_W_S()	{ 
 	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = CVT_W_S;


	PrefetchExtractfs();
	PrefetchExtractfd();
	

}	


/*
Floating Point Divide
To divide FP values
MIPS32
*/
void	DIV_S()		{ 
	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: div_s %s,%s,%s", MipsCpu.PC, debug_r5900_cop1_reg[CURRENT_FD],debug_r5900_cop1_reg[CURRENT_FS] ,debug_r5900_cop1_reg[CURRENT_FT]);

	if ( ( (*MipsCpu.FPR1_p_float[CURRENT_FS]) != 0.0 ) && ( (*MipsCpu.FPR1_p_float[CURRENT_FT]) == 0.0 )  ) {
        
        if ( ( (*MipsCpu.FPR1_p_float[CURRENT_FS]) != 0.0 ) > 0.0 ) *MipsCpu.FPR1_p_float[CURRENT_FD] = FLT_MAX;
        else *MipsCpu.FPR1_p_float[CURRENT_FD] = FLT_MIN;
    }

	if ( ( (*MipsCpu.FPR1_p_float[CURRENT_FS]) == 0.0 ) && ( (*MipsCpu.FPR1_p_float[CURRENT_FT]) == 0.0 )  ) {
        *MipsCpu.FPR1_p_float[CURRENT_FD] = FLT_MAX;
    }

	if ( ( (*MipsCpu.FPR1_p_float[CURRENT_FS]) != 0.0 ) && ( (*MipsCpu.FPR1_p_float[CURRENT_FT]) != 0.0 )  ) {
	
   		*MipsCpu.FPR1_p_float[CURRENT_FD] = (float)((float)(*MipsCpu.FPR1_p_float[CURRENT_FS]) / (float)(*MipsCpu.FPR1_p_float[CURRENT_FT]));
   
    } 
 

    

	INCREMENT_PC();
} 	

void	pDIV_S()		{ 
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = DIV_S;
	
    PrefetchExtractfd();
	PrefetchExtractfs();
	PrefetchExtractft();

} 	


/*
Load Word to Floating Point LWC1
To load a word from memory to an FPR
MIPS32
*/
void    LWC1()		{		
	_u128	temp;

	if ( debug_op == 1 ) 		{
		sprintf(debugTempText,"%8.8llx: lwc1 %s,%4llx(%s)", MipsCpu.PC, debug_r5900_cop1_reg[CURRENT_FT],CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );
		sprintf(debugTempText,"%8.8llx: %s = %8.8llx ", MipsCpu.PC,debug_r5900_cop1_reg[CURRENT_FT],MipsCpu.FPR1[CURRENT_FT].lo);
	}
	
	//sign_extendedh(offset);
	MipsMemoryRead ( &temp , DATA_W ,  MipsCpu.GPR[CURRENT_BASE].lo , CURRENT_OFFSET ) ;
	MipsCpu.FPR1[CURRENT_FT].lo = (_u32)temp.lo;
	
    INCREMENT_PC();
}

void    pLWC1()		{		
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = LWC1;

	
	PrefetchExtractbase();
	PrefetchExtractft();
	PrefetchExtractoffset();
	
	sign_extendedh( MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset );
	
	

}


void 	MADD_S()	{ MipsCpu.halt = 1; } 
void 	pMADD_S()	{	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MADD_S;
 } 

void 	MADDA_S()	{ MipsCpu.halt = 1; } 
void 	pMADDA_S()	{ 	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MADDA_S;
 }

void 	MAX_S()		{ MipsCpu.halt = 1; } 	
void 	pMAX_S()		{	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MAX_S;
  } 

/*
Move Word From Floating Point
To copy a word from an FPU (CP1) general register to a GPR
MIPS32
*/
void	MFC1()		{ 
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: mfc1 %s,%s", MipsCpu.PC, debug_r5900_reg[CURRENT_RT],debug_r5900_cop1_reg[CURRENT_FS] );

  	MipsCpu.GPR[CURRENT_RT].lo = (_u32)MipsCpu.FPR1[CURRENT_FS].lo;
	sign_extended(MipsCpu.GPR[CURRENT_RT].lo);
 
    INCREMENT_PC();
}

void	pMFC1()		{ 
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MFC1;
	PrefetchExtractfs();
	PrefetchExtractrt();
	

}


void	MIN_S()		{ MipsCpu.halt = 1; } 
void	pMIN_S()		{	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode =MIN_S;
  } 

/*
Floating Point Move
To move an FP value between FPRs
MIPS32
*/
void	MOV_S()		{ 


	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: mov_s %s,%s", MipsCpu.PC, debug_r5900_cop1_reg[CURRENT_FD],debug_r5900_cop1_reg[CURRENT_FS] );




    *MipsCpu.FPR1_p_float[CURRENT_FD] = (float) (*MipsCpu.FPR1_p_float[CURRENT_FS]);


	INCREMENT_PC();
} 

void	pMOV_S()		{ 
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MOV_S;



	PrefetchExtractfs();
	PrefetchExtractfd();
	

} 		


void 	MSUB_S()	{ MipsCpu.halt = 1; } 
void 	pMSUB_S()	{ 	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MSUB_S;
}

void 	MSUBA_S()	{ MipsCpu.halt = 1; } 
void 	pMSUBA_S()	{ 	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MSUBA_S;
} 

/*
Move Word to Floating Point
To copy a word from a GPR to an FPU (CP1) general register
MIPS32
*/
void	MTC1()		{ 
	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: mtc1 %s,%s", MipsCpu.PC, debug_r5900_reg[CURRENT_RT],debug_r5900_cop1_reg[CURRENT_FS] );
	
	MipsCpu.FPR1[CURRENT_FS].lo = (_u32)( MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF );						//Copy the val to the reg
	
	INCREMENT_PC();	
}	

void	pMTC1()		{ 
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MTC1;
	PrefetchExtractfs();
	PrefetchExtractrt();
	

}	


/*
Floating Point Multiply
To multiply FP values
MIPS32
*/
void	MUL_S()		{ 
 
 	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: mul_s %s,%s,%s", MipsCpu.PC, debug_r5900_cop1_reg[CURRENT_FD],debug_r5900_cop1_reg[CURRENT_FS] ,debug_r5900_cop1_reg[CURRENT_FT]);
    
	*MipsCpu.FPR1_p_float[CURRENT_FD] =(float)( (float)(*MipsCpu.FPR1_p_float[CURRENT_FS]) * (float)(*MipsCpu.FPR1_p_float[CURRENT_FT]));
   
 	INCREMENT_PC();
} 

void	pMUL_S()		{ 
 	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MUL_S;
 
  	PrefetchExtractfd();
	PrefetchExtractfs();
	PrefetchExtractft();
	

} 				


void	MULA_S()	{ MipsCpu.halt = 1; } 
void	pMULA_S()	{ 	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MULA_S;
} 

/*
Floating Point Negate
To negate an FP value
MIPS32
*/
void	NEG_S()		{ 	
	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: mov_s %s,%s", MipsCpu.PC, debug_r5900_cop1_reg[CURRENT_FD],debug_r5900_cop1_reg[CURRENT_FS] );
  
 

    *MipsCpu.FPR1_p_float[CURRENT_FD] = (float)(-(*MipsCpu.FPR1_p_float[CURRENT_FS]));



    
	INCREMENT_PC();
}
void	pNEG_S()		{ 	
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = NEG_S;

	PrefetchExtractfs();
	PrefetchExtractfd();
	

}

void	RSQRT_S()	{ MipsCpu.halt = 1; } 
void	pRSQRT_S()	{ 	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = RSQRT_S;
 } 

void	SQRT_S()	{ MipsCpu.halt = 1; } 		
void	pSQRT_S()	{ 	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = SQRT_S;
 } 		

/*
Floating Point Subtract
To subtract FP values
MIPS32
*/
void	SUB_S()		{ 
    
	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: sub_s %s,%s,%s\n", MipsCpu.PC, debug_r5900_cop1_reg[CURRENT_FD],debug_r5900_cop1_reg[CURRENT_FS] ,debug_r5900_cop1_reg[CURRENT_FT]);




	*MipsCpu.FPR1_p_float[CURRENT_FD] = (float)((float)(*MipsCpu.FPR1_p_float[CURRENT_FS]) - (float)(*MipsCpu.FPR1_p_float[CURRENT_FT]));





	INCREMENT_PC();
} 		

void	pSUB_S()		{ 
 	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = SUB_S;
   
	PrefetchExtractfd();
	PrefetchExtractfs();
	PrefetchExtractft();
	

} 		


void 	SUBA_S()	{ MipsCpu.halt = 1; } 
void 	pSUBA_S()	{ 	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = SUBA_S;
 } 

/*
Store Word from Floating Point
To store a word from an FPR to memory
MIPS32
*/
void    SWC1()		{
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sdwc1 fp%d,%4llx(%s)", MipsCpu.PC, CURRENT_FT,CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );

	//sign_extendedh(offset);
	
	MipsMemoryWrite ( MipsCpu.FPR1[CURRENT_FT] , DATA_W ,  MipsCpu.GPR[CURRENT_BASE].lo , CURRENT_OFFSET ) ;

    INCREMENT_PC();
}

void    pSWC1()		{
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = SWC1;
	PrefetchExtractbase();
	PrefetchExtractft();
	PrefetchExtractoffset();
	
	sign_extendedh( MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset );

}



/*****************************************************************************************************************
									COP2 INSTRUCTION SET
*****************************************************************************************************************/

void    LQC2()		{	MipsCpu.halt = 1;}
void    pLQC2()		{	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = LQC2;
	}

void    SQC2()		{	MipsCpu.halt = 1;}
void    pSQC2()		{	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = SQC2;
	}

/*****************************************************************************************************************
								EE SPECIFIC INSTRUCTION SET
*****************************************************************************************************************/

void 	MMI0()		{ MipsCpu.halt = 1; }
void 	pMMI0()		{	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MMI0;
  }
 
void 	MMI2()		{ MipsCpu.halt = 1; } 
void 	pMMI2()		{	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MMI2;
  }

void 	MMI1()		{ MipsCpu.halt = 1; } 
void 	pMMI1()		{ 	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MMI1;
 } 

void 	MMI3()		{ MipsCpu.halt = 1; } 
void 	pMMI3()		{ 	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MMI3;
 } 



void 	DIV1()		{ MipsCpu.halt = 1; }
void 	pDIV1()		{ 	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = DIV1;
 }
 

void 	DIVU1()		{ MipsCpu.halt = 1; } 
void 	pDIVU1()		{	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = DIVU1;
  } 



void    LQ()		{	
	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: lq %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[CURRENT_RT],CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );
	
	//sign_extendedh(offset);
	MipsMemoryRead ( &MipsCpu.GPR[CURRENT_RT] , DATA_QUAD ,  MipsCpu.GPR[CURRENT_BASE].lo , CURRENT_OFFSET ) ;
	sign_extended( MipsCpu.GPR[CURRENT_RT].lo );

    INCREMENT_PC();
}

void    pLQ()		{	
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = LQ;
	PrefetchExtractbase();
	PrefetchExtractrt();
	PrefetchExtractoffset();

	sign_extendedh( MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset );
	

}


void	MADD()		{ MipsCpu.halt = 1; } 
void	pMADD()		{	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MADD;
  } 

void	MADD1()		{ MipsCpu.halt = 1; } 
void	pMADD1()		{ 	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MADD1;
 } 
		
void	MADDU()		{ MipsCpu.halt = 1; } 
void	pMADDU()		{ 	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MADDU;
 } 

void	MADDU1()	{ MipsCpu.halt = 1; } 
void	pMADDU1()	{ 	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MADDU1;
 } 

void 	MFHI1()		{ MipsCpu.halt = 1; } 
void 	pMFHI1()		{ 	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MFHI1;
 } 

void 	MFLO1()		{ MipsCpu.halt = 1; } 
void 	pMFLO1()		{	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MFLO1;
 } 

void 	MFSA()		{ MipsCpu.halt = 1;}
void 	pMFSA()		{	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MFSA;
 }

void 	MTSA()		{ MipsCpu.halt = 1;}
void 	pMTSA()		{ 	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MTSA;
}

void 	MTSAB() 	{ MipsCpu.halt = 1;}
void 	pMTSAB() 	{	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MTSAB;
 }

void 	MTSAH () 	{ MipsCpu.halt = 1;}
void 	pMTSAH () 	{	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MTSAH;
 }


void	MULT1()	{
	_s64 temp;
	
	if ( debug_op == 1 ) sprintf(debugTempText,"%8.8llx: mult1 %s,%s,%s", MipsCpu.PC, debug_r5900_reg[CURRENT_RD] , debug_r5900_reg[CURRENT_RS],debug_r5900_reg[CURRENT_RT] );
	
	temp = (_s64)((_s32)(MipsCpu.GPR[CURRENT_RS].lo & 0xFFFFFFFF )) * ((_s32)( MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF ));
	
	MipsCpu.GPR[CPU_REG_LO].lo = temp & 0xFFFFFFFF;
	MipsCpu.GPR[CPU_REG_HI].lo = temp  >> 32 ;
	
	sign_extended(MipsCpu.GPR[CPU_REG_LO].lo);
	sign_extended(MipsCpu.GPR[CPU_REG_HI].lo);
	
	MipsCpu.GPR[CURRENT_RD].lo = MipsCpu.GPR[CPU_REG_LO].lo ;
	
	INCREMENT_PC();
}

void	pMULT1()	{
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MULT1;

	
	PrefetchExtractrs();
	PrefetchExtractrt();
	PrefetchExtractrd();			
	

}


void 	MULTU1()	{ MipsCpu.halt = 1; } 
void 	pMULTU1()	{ 	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MULTU1;
 } 

void	PLZCW()		{ MipsCpu.halt = 1; } 
void	pPLZCW()		{	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = PLZCW;
  } 

void 	PMFHL()		{ MipsCpu.halt = 1; } 
void 	pPMFHL()		{	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = PMFHL;
  } 

void 	PMTHL()		{ MipsCpu.halt = 1; } 
void 	pPMTHL()		{	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = PMTHL;
 } 

void 	MTHI1()		{ MipsCpu.halt = 1; } 
void 	pMTHI1()		{ 	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MTHI1;
 } 

void 	MTLO1()		{ MipsCpu.halt = 1; } 
void 	pMTLO1()		{ 	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MTLO1;
 } 

void 	PSLLH()		{ MipsCpu.halt = 1; } 
void 	pPSLLH()		{	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = PSLLH;
 } 

void 	PSLLW()		{ MipsCpu.halt = 1; } 
void 	pPSLLW()		{	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = PSLLW;
 } 
	
void 	PSRLH()		{ MipsCpu.halt = 1; } 
void 	pPSRLH()		{ 	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = PSRLH;
 } 

void 	PSRAH()		{ MipsCpu.halt = 1; } 
void 	pPSRAH()		{	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = PSRAH;
  }

void 	PSRLW()		{ MipsCpu.halt = 1; } 
void 	pPSRLW()		{ 	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = PSRLW;
 } 

void 	PSRAW()		{ MipsCpu.halt = 1; } 
void 	pPSRAW()		{ 	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = PSRAW;
 } 

void 	SQ() 		{
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sq %s,%4llx(%s)", MipsCpu.PC, debug_r5900_reg[CURRENT_RT],CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );

	sign_extendedh(CURRENT_OFFSET);
	MipsMemoryWrite ( MipsCpu.GPR[CURRENT_RT] , DATA_QUAD ,  MipsCpu.GPR[CURRENT_BASE].lo , CURRENT_OFFSET ) ;
    INCREMENT_PC();
}

void 	pSQ() 		{
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = SQ;

	PrefetchExtractbase();
	PrefetchExtractrt();
	PrefetchExtractoffset();

	sign_extendedh( MipsCpu.ListPrefetchedOpcode[MipsCpu.IndexListPrefetchedOpcode]->offset );
	

}




/*****************************************************************************************************************
									UNCLASSIFIED INSTRUCTION SET
*****************************************************************************************************************/

/*
Perform Cache Operation
To perform the cache operation specified by op.
MIPS32
*/
void    CACHE( ) {

    if ( debug_op == 1 )   sprintf(debugTempText,"%8.8llx: CACHE", MipsCpu.PC);
    
    INCREMENT_PC();
}

void    pCACHE( ) {
	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = CACHE;

  
}



void	CVT_D()		{ MipsCpu.halt = 1; }			
void	pCVT_D()		{	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = CVT_D;
  }


void	MOVF_fmt()	{ MipsCpu.halt = 1; } 
void	pMOVF_fmt()	{	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MOVF_fmt;
  }
 
void	MOVT_fmt()	{ MipsCpu.halt = 1; }	
void	pMOVT_fmt()	{ 	MipsCpu.ListPrefetchedOpcode[ MipsCpu.IndexListPrefetchedOpcode ]->fct_opcode = MOVT_fmt;
 }


/*****************************************************************************************************************
									SYSCALL INSTRUCTION SET
*****************************************************************************************************************/

void SYSCALL_PrintDebugInfo() {

	printf("%8.8llx : %s 0x%x %d \n", MipsCpu.PC , debug_syscall[(MipsCpu.GPR[CPU_REG_V1].lo & 0xFF)],(_u8) (MipsCpu.GPR[CPU_REG_V1].lo & 0xFF),(char) (MipsCpu.GPR[CPU_REG_V1].lo & 0xFF));
	
}


void SYSCALL_NI() {

	printf("%8.8llx : Syscall_NI ( %x h) s8:%d u8:%d\n",MipsCpu.PC ,(_u8) (MipsCpu.GPR[CPU_REG_V1].lo & 0xFF),(char) (MipsCpu.GPR[CPU_REG_V1].lo & 0xFF),(_u8) (MipsCpu.GPR[CPU_REG_V1].lo & 0xFF));
	
	MipsCpu.halt = 1;
}




void SYSCALL_RFU000_FullReset( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();
	MipsCpu.halt = 1; 
} 




/*
ResetEE	ENTRY_1	8000ad68

*/
void SYSCALL_ResetEE( ) 						{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();
	MipsCpu.halt = 1; 
}  





/*
SetGsCrt 02h
8000a060
Need Implementation
Do nothing
*/
void SYSCALL_SetGsCrt( ) { 

	if ( debug_print_syscall == 1 )	{
		SYSCALL_PrintDebugInfo();
		printf("|a0: %8.8llx, a1: 0x%8.8llx, a2:%8.8llx\n",MipsCpu.GPR[CPU_REG_A0].lo,MipsCpu.GPR[CPU_REG_A1].lo,MipsCpu.GPR[CPU_REG_A2].lo);
	}
	

	MipsCpu.PC = 	MipsCpu.PC + 4;	
} 




void SYSCALL_RFU003( ) { 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 
 
void SYSCALL_Exit( ) 							{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 


/*
RFU005	ENTRY_5	80002880
*/ 
void SYSCALL_RFU005( ) 							{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}


/*
LoadPS2Exe	ENTRY_6	800055a0

*/  
void SYSCALL_LoadPS2Exe( ) 						{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 


/*
ExecPS2	ENTRY_7	80002f80


*/ 
void SYSCALL_ExecPS2( ) 						{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}

/*
RFU008	ENTRY_8

*/  
void SYSCALL_RFU008( ) 							{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}  

/*
RFU009	ENTRY_9	80005b28
*/
void SYSCALL_RFU009( ) 							{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}  

/*
AddSbusIntcHandler	ENTRY_10	80001e78

*/
void SYSCALL_AddSbusIntcHandler( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}

/*
RemoveSbusIntcHandler	ENTRY_11	80001ed0


*/  
void SYSCALL_RemoveSbusIntcHandler( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}

/*
Interrupt2Iop	ENTRY_12	80001f70

*/  
void SYSCALL_Interrupt2Iop( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}


/*
SetVTLBRefillHandler	ENTRY_13	800009c0


*/  
void SYSCALL_SetVTLBRefillHandler( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}

/*
SetVCommonHandler	ENTRY_14	80000a00


*/  
void SYSCALL_SetVCommonHandler( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}  

/*
SetVInterruptHandler	ENTRY_15	80000a40


*/
void SYSCALL_SetVInterruptHandler( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}  

/*
AddIntcHandler	ENTRY_16	80001a30


*/
void SYSCALL_AddIntcHandler( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}  

/*
RemoveIntcHandler	ENTRY_17	80001a70


*/
void SYSCALL_RemoveIntcHandler( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}  


/*
AddDmacHandler	ENTRY_18	80001c78

a0: channel number
a1: Address handler
a2: position of the handler ( not used yet )

return position
v0 = 0
*/
void SYSCALL_AddDmacHandler( ) {
	_u32   LastOpcode;
	_u32   AddressLastOpcode;
	
	if ( debug_print_syscall == 1 )	{
		SYSCALL_PrintDebugInfo();
		printf("|channel: %2lld, Address: 0x%8.8llx, handler nb:%lld\n",MipsCpu.GPR[CPU_REG_A0].lo,MipsCpu.GPR[CPU_REG_A1].lo,MipsCpu.GPR[CPU_REG_A2].lo);
	}
	
    //Create a new DMA Handler for the specified Channel
    Ps2Bios.DmaH[ ( MipsCpu.GPR[CPU_REG_A0].lo ) ].Add = MipsCpu.GPR[CPU_REG_A1].lo ;

    LastOpcode = 0;
    AddressLastOpcode = Ps2Bios.DmaH[  ( MipsCpu.GPR[CPU_REG_A0].lo ) ].Add;
  
    while ( LastOpcode != 0x03E00008 ) {
            LastOpcode=*( (_u32 *) (( _u8 *)( MipsMemory + AddressLastOpcode ) ));
            AddressLastOpcode = AddressLastOpcode + 4;
    }
    
    Ps2Bios.DmaH[ ( MipsCpu.GPR[CPU_REG_A0].lo ) ].EndAdd = AddressLastOpcode;
	
	MipsCpu.GPR[CPU_REG_V0].lo = 0 ;
	
	MipsCpu.PC = MipsCpu.PC + 4;
} 

/*
RemoveDmacHandler	ENTRY_19	80001cb8


*/


void SYSCALL_RemoveDmacHandler( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}   

/*
_EnableIntc	ENTRY_20	800008c0


*/

void SYSCALL__EnableIntc( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}   

/*

_DisableIntc	ENTRY_21	80000900


*/
void SYSCALL__DisableIntc( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}   


/*
_EnableDmac	
a0 : Channel Number

return
v0 : 
*/
void SYSCALL__EnableDmac( )  					{

	_u128 dmac_stat;
    _u32  mask;

	if ( debug_print_syscall == 1 )	{
		SYSCALL_PrintDebugInfo();
		printf("|Channel: %lld\n",MipsCpu.GPR[CPU_REG_A0].lo);
	}
	
	//Read the Dmac channel Interrupt Status register
	MipsMemoryRead ( &dmac_stat , DATA_W ,  DMA_STAT  ,0 );
	DmaUpdate();
	
	//Create the MASK concerning the Specified Channel
    mask = 0x10000 << MipsCpu.GPR[CPU_REG_A0].lo ;
    
    //If the Interrupt is already Enable
    if ( (dmac_stat.lo & mask) == mask ) {
         MipsCpu.GPR[CPU_REG_V0].lo =  0;
     }
    //Else enable the Interrupt
    else {
        dmac_stat.lo = dmac_stat.lo | mask ;
        MipsMemoryWrite ( dmac_stat  , DATA_W ,  DMA_STAT  ,0 ) ;
        DmaUpdate();
        MipsCpu.GPR[CPU_REG_V0].lo =  1;
    }
 		
	MipsCpu.PC = MipsCpu.PC + 4;

} 


/*
_DisableDmac	ENTRY_23	80000980


*/
void SYSCALL__DisableDmac( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}

/*
_SetAlarm	ENTRY_24	800022d8


*/ 
void SYSCALL__SetAlarm( ) 						{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 

/*
_ReleaseAlarm	ENTRY_25	80002570

*/
void SYSCALL__ReleaseAlarm( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 


/*
CreateThread	ENTRY_32	80003c50


*/
void SYSCALL_CreateThread( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}

/*
DeleteThread	ENTRY_33	80002fc0


*/ 
void SYSCALL_DeleteThread( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}

/*
StartThread	ENTRY_34	80003040


*/ 
void SYSCALL_StartThread( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}


/*
ExitThread	ENTRY_35	800030c0


*/ 
void SYSCALL_ExitThread( ) 						{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 

/*
ExitDeleteThread	ENTRY_36	80003100


*/
void SYSCALL_ExitDeleteThread( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}

/*
TerminateThread	ENTRY_37	80003140
*/
 
void SYSCALL_TerminateThread( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 

/*
DisableDispatchThread	ENTRY_39	80004228

*/
void SYSCALL_DisableDispatchThread( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 

/*
EnableDispatchThread	ENTRY_40	80004258

*/
void SYSCALL_EnableDispatchThread( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 



void SYSCALL_ChangeThreadPriority( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 




void SYSCALL_RotateThreadReadyQueue( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 




void SYSCALL_ReleaseWaitThread( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 




void SYSCALL_GetThreadId( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 



void SYSCALL_ReferThreadStatus( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 



void SYSCALL_SleepThread( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 



void SYSCALL_WakeupThread( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 



void SYSCALL_CancelWakeupThread( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 



void SYSCALL_SuspendThread( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 



void SYSCALL_ResumeThread( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 



void SYSCALL_JoinThread( ) 						{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
} 

/*

*/
void	SYSCALL_RFU060() {
	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();		
	MipsCpu.GPR[CPU_REG_V0].lo = MipsCpu.GPR[CPU_REG_SP].lo ;				
	MipsCpu.PC = MipsCpu.PC + 4;

}

/*


*/
void	SYSCALL_RFU061() {
	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();			
	MipsCpu.GPR[CPU_REG_V0].lo =	MipsCpu.GPR[CPU_REG_A0].lo + 	MipsCpu.GPR[CPU_REG_A1].lo ;	
	MipsCpu.PC = MipsCpu.PC + 4;

}




void SYSCALL_EndOfHeap( ) 						{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}

  



void SYSCALL_RFU063( ) 							{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();MipsCpu.halt = 1; 
}  

typedef struct _SemaParam {
    int currentCount;               //Current semaphore count
    int maxCount;                   //Maximum semaphore count
    int initCount;                  //Initial semaphore count
    int numWaitThreads;             //Number of threads waiting on semaphore
    int attr;                       //Semaphore attribute
    u_int option;                   //
} SemaParam;

/*
a0: add of sema param
*/
void SYSCALL_CreateSema( ) 						{ 
	
    SemaParam  *mysm;
    _u64    TerminationValue;
   	
   	mysm = (  SemaParam * )	&MipsMemory [  MipsCpu.GPR[CPU_REG_A0].lo & 0x01FFFFFF ];
   	
	if ( debug_print_syscall == 1 )	{
    
    SYSCALL_PrintDebugInfo();
	
	printf("|Address : %llx \n",MipsCpu.GPR[CPU_REG_A0].lo );
    printf("|Sema current: %x\n",    mysm->currentCount);               //Current semaphore count
    printf("|Sema MaxCoun: %x\n",    mysm->maxCount);                   //Maximum semaphore count
    printf("|Sema Initial: %x\n",    mysm->initCount);                  //Initial semaphore count
    printf("|Nb thrds wtg: %x\n",    mysm->numWaitThreads);             //Number of threads waiting on semaphore
    printf("|Sema attribu: %x\n",    mysm->attr);                       //Semaphore attribute
    printf("|Option: %x\n",    mysm->option);                   //

    }

    if ( Ps2Bios.NbSema < BIOS_MAX_SEMAPHORE ) {
    

        Ps2Bios.Sema[ Ps2Bios.NbSema ].MemAddress       = MipsCpu.GPR[CPU_REG_A0].lo;
        Ps2Bios.Sema[ Ps2Bios.NbSema ].current_count    = mysm->currentCount;
        Ps2Bios.Sema[ Ps2Bios.NbSema ].max_count        = mysm->maxCount;
        Ps2Bios.Sema[ Ps2Bios.NbSema ].initial_count    = mysm->initCount;

        Ps2Bios.NbSema = Ps2Bios.NbSema + 1;
        
        TerminationValue = Ps2Bios.NbSema;
        
    
    }
    else TerminationValue = Ps2Bios.NbSema;


    MipsCpu.GPR[CPU_REG_V0].lo = TerminationValue;

	MipsCpu.PC = MipsCpu.PC + 4;	
} 



void SYSCALL_DeleteSema( ) 						{ 

	if ( debug_print_syscall == 1 )	{
        SYSCALL_PrintDebugInfo();	
    	printf("|SemaID : %llx (NI)\n",MipsCpu.GPR[CPU_REG_A0].lo );
	}
 
    MipsCpu.GPR[CPU_REG_V0].lo = MipsCpu.GPR[CPU_REG_A0].lo;
 
    MipsCpu.PC = MipsCpu.PC + 4;
}  



void SYSCALL_SignalSema( ) 						{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	
    MipsCpu.halt = 1; 
}   



void SYSCALL_WaitSema( ) 						{ 

	if ( debug_print_syscall == 1 )	{
        SYSCALL_PrintDebugInfo();	
    	printf("|SemaID : %llx (NI)\n",MipsCpu.GPR[CPU_REG_A0].lo );
	}
 
    MipsCpu.GPR[CPU_REG_V0].lo = MipsCpu.GPR[CPU_REG_A0].lo;
 
    MipsCpu.PC = MipsCpu.PC + 4;
}   



void SYSCALL_PollSema( ) 						{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_ReferSemaStatus( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_RFU073( ) 							{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_SetOsdConfigParam( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_GetOsdConfigParam( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_GetGsHParam( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}  



void SYSCALL_GetGsVParam( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_SetGsHParam( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_SetGsVParam( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_RFU080_CreateEventFlag( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_RFU081_DeleteEventFlag( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_RFU082_SetEventFlag( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_RFU083_iSetEventFlag( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_RFU084_ClearEventFlag( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_RFU085_iClearEventFlag( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_RFU086_WaitEvnetFlag( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_RFU087_PollEvnetFlag( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_RFU088_iPollEvnetFlag( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_RFU089_ReferEventFlagStatus( ) 	{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_RFU091( ) 							{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_EnableIntcHandler( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_DisableIntcHandler( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_EnableDmacHandler( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_DisableDmacHandler( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_KSeg0( ) 							{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_EnableCache( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}  



void SYSCALL_DisableCache( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_GetCop0( ) 						{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   

/*
FlushCache 0x64 100
Do Nothing
*/
void SYSCALL_FlushCache( ) 						{ 
    static _u64 NbFlushCache = 0;;

	if ( debug_print_syscall == 1 )	{
//        SYSCALL_PrintDebugInfo();	
    printf("FlushCache : n %5.5lld \n",NbFlushCache);

    }
	NbFlushCache++;
    MipsCpu.PC = MipsCpu.PC + 4;
}   



void SYSCALL_CpuConfig( ) 						{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_sceSifStopDma( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_SetCPUTimerHandler( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_SetCPUTimer( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_ForceRead( ) 						{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   



void SYSCALL_ForceWrite( ) 						{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}   

/*

*/
void SYSCALL_GsGetIMR( ) 						{ 
	if ( debug_print_syscall == 1 )	{ 
		SYSCALL_PrintDebugInfo();
		printf("|???\n");
	}	
	
	MipsCpu.PC = MipsCpu.PC + 4;
} 

/*
GsPutIMR
Write A0 to 0x12001010
V0 = A0
*/
void SYSCALL_GsPutIMR( ) 						{ 
	if ( debug_print_syscall == 1 )	{
		SYSCALL_PrintDebugInfo();		
		printf("|A0	= %llx\n",MipsCpu.GPR[CPU_REG_A0].lo);
	}
	
	
	MipsCpu.GPR[CPU_REG_V0].lo = MipsCpu.GPR[CPU_REG_A0].lo;
	MipsMemoryWrite ( MipsCpu.GPR[CPU_REG_A0] , DATA_DW ,  0x12001010  ,0 ) ;			// (12001010 : GS_IMR) = A0
	
	
	
	MipsCpu.PC = MipsCpu.PC + 4;
} 




void SYSCALL_SetPgifHandler( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}    



void SYSCALL_SetVSyncFlag( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}    



void SYSCALL_RFU116( ) 							{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	
    MipsCpu.halt = 1; 
    //	MipsCpu.PC = MipsCpu.PC + 4;
}    


void SYSCALL_print( ) 							{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}    



void SYSCALL_sceSifDmaStat( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}    

typedef struct _SifDmaData{
_u32 data; 
_u32 addr; 
_u32 size; 
_u32 mode;
} SifDmaData;


void SYSCALL_sceSifSetDma( ) 					{ 

   SifDmaData *sdd;
   
   _u8	boucle;
   _u16	* ReadData  ;
 
 	sdd = (  SifDmaData * )	&MipsMemory [  MipsCpu.GPR[CPU_REG_A0].lo & 0x01FFFFFF ];
 
 	if ( debug_print_syscall == 1 )	{
		SYSCALL_PrintDebugInfo();

		printf("|addr=%16.16llx, nb=%llx \n",MipsCpu.GPR[CPU_REG_A0].lo,MipsCpu.GPR[CPU_REG_A1].lo);
        printf("|data:%8.8lx\n",sdd->data);
        printf("|addr:%8.8lx\n",sdd->addr);
    	printf("|size:%8.8lx\n",sdd->size);
    	printf("|mode:%8.8lx\n",sdd->mode);
    	/*
    	printf("Data at Adrr: ");
    	for ( boucle = 0 ; boucle < sdd->size ; boucle++ ) {
 
    		ReadData = (  _u16 * )&MipsMemory [  ((sdd->data)&0x00FFFFFF) + ( boucle * 2 )  ];
    		
    		printf("%4.4x ",*ReadData);
    		
    	}	
    	printf("\n");
    	*/
    	//getchar();	
    	
    	

    	
    	
    	
	}
		

    if ( Ps2Bios.DmaH[ DMA_CH5 ].Enable == TRUE ) {

        MipsCpu.GPR[CPU_REG_V0].lo = 1 ;

        memcpy( Ps2Bios.SaveReg , MipsCpu.GPR , (sizeof(_u128)) * 34 );
         
    
        Ps2Bios.DmaH[ DMA_CH5 ].ReturnAdd =  MipsCpu.PC + 4;
        Ps2Bios.MustExecHandler = DMA_CH5 ;
        MipsCpu.PC = Ps2Bios.DmaH[ DMA_CH5 ].Add - 4;
        
        printf("Exec Handler at %8.8llx",Ps2Bios.DmaH[ DMA_CH5 ].Add);
        printf("--->  %8.8lx \n",Ps2Bios.DmaH[ DMA_CH5 ].EndAdd);
 
 
        //debug_op = 1;
        //MipsCpu.halt = 1;

 
 
    }

 
    MipsCpu.GPR[CPU_REG_V0].lo = 1 ;
	MipsCpu.PC = MipsCpu.PC + 4;
} 


/*
#---------------------------------------------------------------------------------------
#isceSifSetDChain
#---------------------------------------------------------------------------------------
*/
void SYSCALL_sceSifSetDChain( )  				{	
	_u128 temp;
	
	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();		

	MipsCpu.PC = MipsCpu.PC + 4;
} 
 


void SYSCALL_sceSifSetReg( ) 					{ 


	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     

/*
sceSifGetReg
a0 ( _u16 ) : Register number
*/
void SYSCALL_sceSifGetReg( ) 					{
	
	if ( debug_print_syscall == 1 ) {
		SYSCALL_PrintDebugInfo();
    	printf("|a0=%llx\n", MipsCpu.GPR[CPU_REG_A0].lo);
	}
	
	MipsCpu.GPR[CPU_REG_V0].lo = 0x00060000;
	MipsCpu.PC = MipsCpu.PC + 4;
} 
 



void SYSCALL_ExecOSD( ) 						{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     



void SYSCALL_Deci2Call( ) 						{ 

	if ( debug_print_syscall == 1 ) SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     



void SYSCALL_PSMode( ) 							{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     



void SYSCALL_MachineType( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     



void SYSCALL_GetMemorySize( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     



void SYSCALL_isceSifSetDChain( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     



void SYSCALL_isceSifSetDma( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     



void SYSCALL_isceSifDmaStat( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


/*
GsPutIMR	ENTRY_113	80000d40
12001010	IMR			Interrupt mask

*/
void SYSCALL_iGsPutIMR( ) 						{ 
	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();
	MipsCpu.PC = MipsCpu.PC + 4;
} 


/*
GsGetIMR	ENTRY_112	80000d00



*/
void SYSCALL_iGsGetIMR( ) 						{

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();
	MipsCpu.PC = MipsCpu.PC + 4;
} 



void SYSCALL_iCpuConfig( ) 						{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL_iFlushCache( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL_iGetCop0( ) 						{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL_iDisableDmacHandler( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL_iEnableDmacHandler( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL_iDisableIntcHandler( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL_iEnableIntcHandler( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL_RFU090_iReferEventFlagStatus( )	{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL_iReferSemaStatus( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL_iPollSema( ) 						{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     



void SYSCALL_iSignalSema( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL_iResumeThread( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL_iSuspendThread( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL_iCancelWakeupThread( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL_iWakeupThread( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL_iReferThreadStatus( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL_iReleaseWaitThread( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL_iRotateThreadReadyQueue( )			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL_iChangeThreadPriority( ) 			{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}    


void SYSCALL_iTerminateThread( ) 				{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL__iReleaseAlarm( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL__iSetAlarm( ) 						{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL__iDisableDmac( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
} 


void SYSCALL__iEnableDmac( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL__iDisableIntc( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL__iEnableIntc( ) 					{ 

	if ( debug_print_syscall == 1 )	SYSCALL_PrintDebugInfo();	MipsCpu.halt = 1; 
}     


void SYSCALL_UKN_1() { 
	
	if ( debug_print_syscall == 1 )	{
		SYSCALL_PrintDebugInfo();	
		printf("|???\n");
	}
	
	//MipsCpu.halt = 1;
	MipsCpu.PC = MipsCpu.PC + 4; 
}    


